# Sacha van Albada, Nov 2014
# Based on code by Moritz Helias

# Simulation script for binary network with population-dependent connection probability

import nest
import numpy


from parameters_binary_asymm import *

print 'scale type:', scale_type
print 'N_scaling:', N_scaling
if scale_K_equally:
    print 'K_scaling:', K_scaling
print 'J_array:', J_array
print 'K_array:', K_array
print 'mu_x:', mu_x
print 'sigma_x:', sigma_x

suffix="asymm_ext"

nest.ResetKernel()
nest.SetKernelStatus({"resolution": h, "print_time": False, "overwrite_files": True, 'total_num_virtual_procs': n_virtual_procs, "grng_seed": msd+n_virtual_procs, "rng_seeds" : range(msd+n_virtual_procs+1, msd+2*n_virtual_procs+1)})


# this line is crucial to prevent multapses from being created
# otherwise the decoding of up-/down transitions will go wrong
nest.sr("/RandomConvergentConnect << /allow_multapses false >> SetOptions")


nest.SetDefaults("multimeter",
                {"withtime": False,
                 "withgid": False,
                 "to_memory": False,
                 "to_file": True,
                 "record_from": ["h"],
                 "interval": dt_samp})

# create recorders for activity
multi_exc1 = nest.Create("multimeter")
multi_exc2 = nest.Create("multimeter")
multi_inh1 = nest.Create("multimeter")
multi_inh2 = nest.Create("multimeter")
if input_type == 'binary':
    multi_ext = nest.Create("multimeter")

print "Building network"

# take into account missing external input
# by shifting the threshold by the mean that
# the external input would provide

if input_type == 'binary':
    # mean activity m_x
    nest.SetDefaults('ginzburg_neuron', {'theta': 0.0, 'tau_m': tau_m, 'c_1': 0.0, 'c_2': 2.*m_x, 'c_3': 1.0})
    # excitatory neurons
    nrnX = nest.Create('ginzburg_neuron', N)

if input_type == 'noise':
    ngE = nest.Create('noise_generator')
    ngI = nest.Create('noise_generator')
    nest.SetStatus(ngE, {'mean': noise_mean_E, 'std': noise_std_E, 'dt': 0.1})
    nest.SetStatus(ngI, {'mean': noise_mean_I, 'std': noise_std_I, 'dt': 0.1})

if kickstart:
    # for kickstarting an otherwise silent network
    ks = nest.Create('noise_generator')
    nest.SetStatus(ks, {'mean': ks_mean, 'std': ks_std, 'stop': ks_stop, 'dt': 0.1})

if neuronmodel == 'ginzburg_neuron':
    nest.SetDefaults('ginzburg_neuron', {'theta': theta, 'tau_m': tau_m, 'c_1': 0.0, 'c_2': 1.0, 'c_3': beta})
if neuronmodel == 'mcculloch_pitts_neuron':
    nest.SetDefaults('mcculloch_pitts_neuron', {'theta': theta, 'tau_m': tau_m})

# excitatory neurons
nrnE = nest.Create(neuronmodel, N)

# inhibitory neurons
nrnI = nest.Create(neuronmodel, N)

# these neurons are only used to record the activity of subpopulations in the network
nrn_pre_sum_exc1 = nest.Create(neuronmodel)
nrn_pre_sum_exc2 = nest.Create(neuronmodel)

nrn_pre_sum_inh1 = nest.Create(neuronmodel)
nrn_pre_sum_inh2 = nest.Create(neuronmodel)

if record_spin:
    # for recording the activity of some single neurons in order to determine 
    # average auto-correlations
    sd_e = nest.Create('spin_detector')
    sd_i = nest.Create('spin_detector')

if input_type == 'binary':
    nrn_pre_sum_ext = nest.Create(neuronmodel)

nest.SetStatus(multi_exc1, {"label": datapath + "binary-exc1" + suffix})

nest.SetStatus(multi_exc2, {"label": datapath + "binary-exc2" + suffix})

nest.SetStatus(multi_inh1, {"label": datapath + "binary-inh1" + suffix})

nest.SetStatus(multi_inh2, {"label": datapath + "binary-inh2" + suffix})

if input_type == 'binary':
    nest.SetStatus(multi_ext, {"label": datapath + "binary-ext" + suffix})

if record_spin:
    nest.SetStatus(sd_e, {"to_file": True, "label": datapath + "binary-sd_e" + suffix})
    nest.SetStatus(sd_i, {"to_file": True, "label": datapath + "binary-sd_i" + suffix})

print "Creating recurrent connectivity"

numpy.random.seed(1)

# recurrent connectivity from excitatory to excitatory neurons
nest.SetDefaults('static_synapse', {'weight': j_EE, 'delay': d, 'receptor_type': 0})

nest.RandomConvergentConnect(nrnE, nrnE, K_EE)

# recurrent connectivity from inhibitory to excitatory neurons
nest.SetDefaults('static_synapse', {'weight': j_EI, 'delay': d, 'receptor_type': 0})

nest.RandomConvergentConnect(nrnI, nrnE, K_EI)

# recurrent connectivity from excitatory to inhibitory neurons
nest.SetDefaults('static_synapse', {'weight': j_IE, 'delay': d, 'receptor_type': 0})

nest.RandomConvergentConnect(nrnE, nrnI, K_IE)

# recurrent connectivity from inhibitry to inhibitory neurons
nest.SetDefaults('static_synapse', {'weight': j_II, 'delay': d, 'receptor_type': 0})

nest.RandomConvergentConnect(nrnI, nrnI, K_II)

if input_type == 'binary':
    # connectivity from external to excitatory neurons
    nest.SetDefaults('static_synapse', {'weight': j_EX, 'delay': d, 'receptor_type': 0})
    nest.RandomConvergentConnect(nrnX, nrnE, K_EX)
    # connectivity from external to inhibitory neurons
    nest.SetDefaults('static_synapse', {'weight': j_IX, 'delay': d, 'receptor_type': 0})
    nest.RandomConvergentConnect(nrnX, nrnI, K_IX)

if input_type == 'noise':
    nest.DivergentConnect(ngE, nrnE, 1.0, 0.1)
    nest.DivergentConnect(ngI, nrnI, 1.0, 0.1)

if kickstart:
    nest.DivergentConnect(ks, nrnE, 1.0, 0.1)

print "Connecting devices."

nest.SetDefaults('static_synapse', {'weight': 1.0, 'delay': h})

nest.ConvergentConnect(nrnE[:N_rec_E], nrn_pre_sum_exc1)
nest.ConvergentConnect(nrnE[N/2:N/2+N_rec_E], nrn_pre_sum_exc2)

nest.ConvergentConnect(nrnI[:N_rec_I], nrn_pre_sum_inh1)
nest.ConvergentConnect(nrnI[N/2:N/2+N_rec_I], nrn_pre_sum_inh2)

if input_type == 'binary':
    nest.ConvergentConnect(nrnX, nrn_pre_sum_ext)

nest.Connect(multi_exc1, nrn_pre_sum_exc1)
nest.Connect(multi_exc2, nrn_pre_sum_exc2)
nest.Connect(multi_inh1, nrn_pre_sum_inh1)
nest.Connect(multi_inh2, nrn_pre_sum_inh2)
if input_type == 'binary':
    nest.Connect(multi_ext, nrn_pre_sum_ext)

if record_spin:
    nest.ConvergentConnect(nrnE[:n_spin], sd_e)
    nest.ConvergentConnect(nrnI[:n_spin], sd_i)

nest.Simulate(T)



