# Sacha van Albada, Moritz Helias
# Nov 2014

# Framework for network scaling
# Data created using hambach_run_sim_dist_Kin_scale.py

import os.path

import h5py_wrapper as pytables

from parameters_ginzburg import datapath, j_EE, j_EI, j_II, j_IE, j_EX, j_IX, p, theta, m_x

from ginzburg_theory_distKin import ginzburg

from plot_utils import *

import ps_manip as ps

import numpy as np

import matplotlib.pyplot as plt

from matplotlib.pylab import rcParams, savefig

figurepath = '../figures/'
figurename = 'scaling_framework'
suffix = 'infinite'
# markersize
mrk = 6.

i0 = 200 # skip to get rid of initial transient

tmin, tmax = -25., 25.

mycols = ['k', '#1d771d', '#a8d57b']

def loaddata_renart_10000():

    suffix = 'dist_Kin_scale10000'
    m_exc1 = np.loadtxt(datapath + 'ginzburg-exc1'+suffix+'-00001-246.dat')
    m_exc2 = np.loadtxt(datapath + 'ginzburg-exc2'+suffix+'-00002-247.dat')

    m_inh1 = np.loadtxt(datapath + 'ginzburg-inh1'+suffix+'-00003-248.dat')
    m_inh2 = np.loadtxt(datapath + 'ginzburg-inh2'+suffix+'-00004-249.dat')

    m_ext = np.loadtxt(datapath + 'ginzburg-ext'+suffix+'-00005-250.dat')

    return m_exc1, m_exc2, m_inh1, m_inh2, m_ext


def loaddata_renart_30000():

    suffix = 'dist_Kin_scale30000'
    m_exc1 = np.loadtxt(datapath + 'ginzburg-exc1'+suffix+'-00001-246.dat')
    m_exc2 = np.loadtxt(datapath + 'ginzburg-exc2'+suffix+'-00002-247.dat')

    m_inh1 = np.loadtxt(datapath + 'ginzburg-inh1'+suffix+'-00003-248.dat')
    m_inh2 = np.loadtxt(datapath + 'ginzburg-inh2'+suffix+'-00004-249.dat')

    m_ext = np.loadtxt(datapath + 'ginzburg-ext'+suffix+'-00005-250.dat')

    return m_exc1, m_exc2, m_inh1, m_inh2, m_ext


def loaddata_renart_100000():

    suffix = 'dist_Kin_scale100000'
    m_exc1 = np.loadtxt(datapath + 'ginzburg-exc1'+suffix+'-000001-006.dat')
    m_exc2 = np.loadtxt(datapath + 'ginzburg-exc2'+suffix+'-000002-007.dat')

    m_inh1 = np.loadtxt(datapath + 'ginzburg-inh1'+suffix+'-000003-008.dat')
    m_inh2 = np.loadtxt(datapath + 'ginzburg-inh2'+suffix+'-000004-009.dat')

    m_ext = np.loadtxt(datapath + 'ginzburg-ext'+suffix+'-000005-010.dat')

    return m_exc1, m_exc2, m_inh1, m_inh2, m_ext




def correlate(N_rec, suffix, loader, date_checker):

    # check if data is newer than cache
    filename_cache = datapath + 'corr_cache_'+ suffix + '.h5'
    requires_recalc = False
    if os.path.isfile(filename_cache):
        requires_recalc =  os.path.getmtime(datapath + date_checker) > os.path.getmtime(filename_cache)
    else:
        requires_recalc = True


    if requires_recalc:

        m_exc1, m_exc2, m_inh1, m_inh2, m_ext = loader()

        m_exc1 = m_exc1[i0:]
        m_exc2 = m_exc2[i0:]
        m_inh1 = m_inh1[i0:]
        m_inh2 = m_inh2[i0:]
        m_ext = m_ext[i0:]

        m_exc = m_exc1 + m_exc2
        m_inh = m_inh1 + m_inh2

        mean_exc1 = np.mean(m_exc1)
        mean_inh1 = np.mean(m_inh1)
        mean_exc2 = np.mean(m_exc2)
        mean_inh2 = np.mean(m_inh2)

        mean_exc = np.mean(m_exc)
        mean_inh = np.mean(m_inh)
        mean_ext = np.mean(m_ext)

        cc_ee = 1./(len(m_exc1)*N_rec**2) * np.correlate(m_exc1-mean_exc1, m_exc2-mean_exc2, mode='full')

        cc_ei = 1./(len(m_exc1)*N_rec**2) * np.correlate(m_exc1-mean_exc1, m_inh1-mean_inh1, mode='full')

        cc_ii = 1./(len(m_exc1)*N_rec**2) * np.correlate(m_inh1-mean_inh1, m_inh2-mean_inh2, mode='full')

        cc_ex = 1./(len(m_exc1)*N_rec*N) * np.correlate(m_exc1-mean_exc1, m_ext-mean_ext, mode='full')

        cc_ix = 1./(len(m_exc1)*N_rec*N) * np.correlate(m_inh1-mean_inh1, m_ext-mean_ext, mode='full')

        a_e = 1./(len(m_exc1)*N_rec) * np.correlate(m_exc1-mean_exc1, m_exc1-mean_exc1, mode='full') - cc_ee*N_rec

        a_i = 1./(len(m_exc1)*N_rec) * np.correlate(m_inh1-mean_inh1, m_inh1-mean_inh1, mode='full') - cc_ii*N_rec

        a_x = 1./(len(m_exc1)*N) * np.correlate(m_ext-mean_ext, m_ext-mean_ext, mode='full')

        C_e = 1./(len(m_exc)*(2.*N_rec)**2) * np.correlate(m_exc-mean_exc, m_exc-mean_exc, mode='full')
        C_i = 1./(len(m_inh)*(2.*N_rec)**2) * np.correlate(m_inh-mean_inh, m_inh-mean_inh, mode='full')
        C_ei = 1./(len(m_exc)*(2.*N_rec)**2) * np.correlate(m_exc-mean_exc, m_inh-mean_inh, mode='full')

        pytables.add_to_h5(filename_cache, { 'c_ee': cc_ee, 'c_ei': cc_ei, 'c_ii': cc_ii, 'c_ex': cc_ex, 'c_ix': cc_ix, 'a_e': a_e, 'a_i': a_i, 'a_x': a_x, 'C_e': C_e, 'C_i': C_i, 'C_ei': C_ei, 'm_e': [mean_exc], 'm_i': [mean_inh] })


    corrdata = pytables.load_h5(filename_cache)

    return corrdata



def correlations_renart(N, m_E, m_I, m_X, q_E, q_I):

    invdet = -1./(j_EE*j_II - j_EI*j_IE)
    print "invdet = ", invdet
    A_E = invdet * (j_II*j_EX - j_EI*j_IX) 
    A_I = invdet * (-j_IE*j_EX + j_EE*j_IX) 

    print "A_E, A_I =", A_E, A_I 

    a_X = m_X*(1.-m_X)
    a_E = m_E - q_E
    a_I = m_I - q_I
    print 'a_E(q) = ', a_E
    print 'a_I(q) = ', a_I

    c_ee = 1./N * (A_E**2 * a_X - a_E)
    c_ii = 1./N * (A_I**2 * a_X - a_I)
    c_ei = 1./N * (A_E * A_I * a_X)

    c_ex = 1./N * A_E**2 * a_X
    c_ix = 1./N * A_I**2 * a_X

    return c_ee, c_ii, c_ei, c_ex, c_ix


    

def cmp_accuracy(N, corrdata):

    mid_index = len(corrdata['c_ee'])/2
    print mid_index
    c_ee = corrdata['c_ee'][mid_index]
    c_ii = corrdata['c_ii'][mid_index]
    c_ei = corrdata['c_ei'][mid_index]
    print ' c_ee (sim) = ', c_ee
    print ' c_ii (sim) = ', c_ii
    print ' c_ei (sim) = ', c_ei

    # our theory
    J = np.array([[j_EE, j_EI, j_EX], [j_IE, j_II, j_IX]]) * np.sqrt(1.0*N0/N)
    g = ginzburg(np.array([N,N,N]), J, p, m_x, theta)
    m = g.solve_self_consistently(np.array([m_x, m_x]))

    m1, q1 = g.solve_self_consistently_m2(m, m*m)
    S = g.susceptibility_m2_corr(m1, q1, np.zeros(5))   # assume zero correlation here
    c0, c1 = g.correlations1(np.hstack((m1, m_x)), np.hstack((q1, m_x*m_x)), S)

    print "m, q =", m1, q1
    c_th = c0 + c1
    print "c(theo) = ", c_th
    print "c(sim) = ", c_ee, c_ei, c_ii
    print "rel errors (ee, ei, ii)", (c_th[0]-c_ee)/c_ee, (c_th[1]-c_ei)/c_ei, (c_th[2]-c_ii)/c_ii

    # Renart's theory

    print 'first and second moment', m1, q1
    c_r = np.array(correlations_renart(N, m1[0], m1[1], m_x, q1[0], q1[1]))
    print c_r
    print "rel errors renart (ee, ei, ii)", (c_r[0]-c_ee)/c_ee, (c_r[1]-c_ii)/c_ii, (c_r[2]-c_ei)/c_ei

##############
# simulations
##############

CS_dist_10000 = correlate(5000, 'dist_Kin_scale10000', loaddata_renart_10000, 'ginzburg-exc1dist_Kin_scale10000-00001-246.dat')

CS_dist_30000 = correlate(15000, 'dist_Kin_scale30000', loaddata_renart_30000, 'ginzburg-exc1dist_Kin_scale30000-00001-246.dat')

CS_dist_100000 = correlate(50000, 'dist_Kin_scale100000', loaddata_renart_100000, 'ginzburg-exc1dist_Kin_scale100000-000001-006.dat')

N0 = 2**13
Ns = 2**np.arange(10., 35., 1.)


cmp_accuracy(10000, CS_dist_10000)
cmp_accuracy(30000, CS_dist_30000)
cmp_accuracy(100000, CS_dist_100000)

ms = []
cs0 = []
cs1 = []
cs_r = []

m_x = 0.1

for N in Ns:
    print 'calculating for N=', N

    # our theory
    J = np.array([[j_EE, j_EI, j_EX], [j_IE, j_II, j_IX]]) * np.sqrt(1.0*N0/N)
    g = ginzburg(np.array([N,N,N]), J, p, m_x, theta)
    m = g.solve_self_consistently(np.array([m_x, m_x]))

    m1, q1 = g.solve_self_consistently_m2(m, m*m)
    S = g.susceptibility_m2_corr(m1, q1, np.zeros(5))   # assume zero correlation here
    c0, c1 = g.correlations1(np.hstack((m1, m_x)), np.hstack((q1, m_x*m_x)), S)

    print "m, q =", m1, q1
    ms.append(m1)
    cs0.append(c0)
    cs1.append(c1)
    
    # Renart's theory

    print 'first and second moment', m1, q1
    c_r = np.array(correlations_renart(N, m1[0], m1[1], m_x, q1[0], q1[1]))
    cs_r.append(c_r)

ms = np.array(ms)
cs0 = np.array(cs0)
cs1 = np.array(cs1)
cs_r = np.array(cs_r)


##############
# plot using plot_utils
##############

scale=1
width = 6.83

labels = ['A', 'B', 'C', 'D']
panelgrid = [3,2]
rowspan = [[0,1], [0,1], [1,3], [1,3]]      # extent of each panel can be specified in terms of 
colspan = [[0,1], [1,2], [0,1], [1,2]]      # spans in row (vertical) and column (horizontal)
param = ParamFactory().fromfile("../material/fig_scaling_framework.json")

fig = plt.figure(figsize=[scale*width, scale*width/2.])
fig.subplots_adjust(bottom=0.17)
panels = genpanels(fig, param)      # 'panels' contain all axes in the figure

panels['C'][0].plot(Ns, cs1[:,0], color=mycols[1], label=r'$c^\mathrm{int}$')

panels['C'][0].plot(Ns, cs0[:,0] + cs1[:,0], color=mycols[0], label=r'$c^\mathrm{int} + c^\mathrm{ext}$')

panels['C'][0].plot(Ns, cs0[:,0], color=mycols[2], label=r'$c^\mathrm{ext}$')

rcParams['legend.fontsize'] = scale*12

panels['C'][0].legend()

mid_index = len(CS_dist_10000['c_ee'])/2
print mid_index
print ' c_ee (sim) = ', CS_dist_10000['c_ee'][mid_index]
print ' c_ii (sim) = ', CS_dist_10000['c_ii'][mid_index]
print ' c_ei (sim) = ', CS_dist_10000['c_ei'][mid_index]

panels['C'][0].loglog([10000], [CS_dist_10000['c_ee'][mid_index]], 'x', ms=4, color='black')

mid_index = len(CS_dist_30000['c_ee'])/2
print mid_index
print ' c_ee (sim) = ', CS_dist_30000['c_ee'][mid_index]
print ' c_ii (sim) = ', CS_dist_30000['c_ii'][mid_index]
print ' c_ei (sim) = ', CS_dist_30000['c_ei'][mid_index]

panels['C'][0].loglog([30000], [CS_dist_30000['c_ee'][mid_index]], 'x', ms=4, color='black')

mid_index = len(CS_dist_100000['c_ee'])/2
print mid_index
print ' c_ee (sim) = ', CS_dist_100000['c_ee'][mid_index]
print ' c_ii (sim) = ', CS_dist_100000['c_ii'][mid_index]
print ' c_ei (sim) = ', CS_dist_100000['c_ei'][mid_index]

panels['C'][0].loglog([100000], [CS_dist_100000['c_ee'][mid_index]], 'x', ms=4, color='black')

setparams(panels, param)
draw_labels(panels, param)

savefig(figurepath + figurename + '.eps')

ps.merge_inkscape_eps_into_matplotlib_eps(figurepath + figurename + '.eps', figurepath + 'why_scaling.eps', figurepath + figurename + '_mrgd1.eps', (62., 189.), (0.32, 0.32))

ps.merge_inkscape_eps_into_matplotlib_eps(figurepath + figurename + '_mrgd1.eps', figurepath + 'relevant_scales.eps', figurepath + figurename + '_mrgd2.eps', (290., 180.), (0.32, 0.32))

ps.merge_inkscape_eps_into_matplotlib_eps(figurepath + figurename + '_mrgd2.eps', figurepath + 'scaling_as_subsampling.eps', figurepath + 'fig1.eps', (300., 10.), (0.55, 0.55))



