# Sacha van Albada
# Nov 2014

# Scaling of single-population inhibitory LIF network
# Panel A: pole structure 
# Panel B: covariance functions
# Data created using hambach_run_inhibitory_LIF_network.py

import siegert as sig
import numpy as np
from plotfuncs import *
from pylab import *
from corr_hawkes_like import *
from parameters_inhibitory_LIF_network import *

figurepath = '../figures/'
datapath = '../data/spiking/inhibitory_LIF_network/'
suffix = '.dat'

mycols = [black, '#1d771d', '#a8d57b']

# initialize corr_hawkes_like object as if we had an E-I network;
# only delay and tau_eff are used for the calculation of the poles
gamma = 1.
g = 1.
NE = 1.
w = 1.
nu = 1.
alpha = 1.
poles_max = 1.
hawkes = corr_hawkes_like(tau_eff, delay, gamma, g, epsilon, NE, w, nu, sigma_a, alpha, poles_max)

N0 = 10000
N = 50000
K0 = int(epsilon*N0)
K = int(epsilon*N)
# eigenvalue of connectivity in reference network
# mu and sigma are mean and standard deviation of inputs to the neurons in mV
L0 = K0 * sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, -J0)[0]
L_vreeswijk = K * sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, -J0*np.sqrt(K0/float(K)))[0]
L_helias = K * sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, -J0*K0/float(K))[0]

branch_max = 5

poles0 = hawkes.get_poles_(L0, branch_max)
poles_vreeswijk = hawkes.get_poles_(L_vreeswijk, branch_max)
poles_helias = hawkes.get_poles_(L_helias, branch_max)

###########################################################

# get correlations

def get_corr_data(filename, n1, n2, T, tmax, tbin):
    """Get normalized correlation data"""
    
    corr_data = np.loadtxt(filename)
    corr = corr_data[2:]
    count1 = corr_data[0]
    count2 = corr_data[1]
    T_eff = T - tmax    # used new feature of correlation detector to start recording after tmax to avoid triangular falloff to the sides
    rate1 = count1 / (n1*T_eff/1000.0)
    rate2 = count2 / (n2*T_eff/1000.0)
    rate = np.sqrt(rate1*rate2)
    print "rate = ", rate
    corr = 1.0*corr/(n1*n2)/T_eff*1000.0/tbin*1000.0 / (rate1*rate2) - 1.

    return corr, rate


def get_corr(T, N_rec, tmax, tbin, datapath, suffix):

    corr_ref, rate_ref = get_corr_data(datapath + 'corr_none' + suffix, N_rec, N_rec, T, tmax, tbin)
    corr_vreeswijk, rate_vreeswijk = get_corr_data(datapath + 'corr_vreeswijk' + suffix, N_rec, N_rec, T, tmax, tbin)
    corr_helias, rate_helias = get_corr_data(datapath + 'corr_helias' + suffix, N_rec, N_rec, T, tmax, tbin)

    return corr_ref, corr_vreeswijk, corr_helias, rate_ref, rate_vreeswijk, rate_helias


###########################################################

# plot

scale = 1.
width = 6.83
n_horz_panels = 2
n_vert_panels = 1

panel_factory = create_fig_plos(1, scale, width, n_horz_panels, n_vert_panels)

axA = panel_factory.new_panel(0, 0, r'A')
plot(imag(poles0[0]), real(poles0[0]), '.', color=mycols[0], markersize=8, label=r'$N=10,\!000$')
plot(imag(poles_vreeswijk[0]), real(poles_vreeswijk[0]), 'x', color=mycols[1], markersize=5, mew=1, label=r'$N=50,\!000, J\propto\frac{1}{\sqrt{K}}$')
plot(imag(poles_helias[0]), real(poles_helias[0]), 'x', color=mycols[2], markersize=5, mew=1, label=r'$N=50,\!000, J\propto\frac{1}{K}$')
# corr_hawkes_like.py gives poles corresponding to i*omega. 
# But in our paper, we have treated poles as values of omega.
# Therefore, [-real(pole), imag(pole)] = [imag(z), real(z)] where z is our usage.
for pole in poles0:
    plot(imag(pole), real(pole), '.', color=mycols[0], markersize=8)
for pole in poles_vreeswijk:
    plot(imag(pole), real(pole), 'x', color=mycols[1], markersize=5, mew=1)
for pole in poles_helias:
    plot(imag(pole), real(pole), 'x', color=mycols[2], markersize=5, mew=1)

fill_between([-30,30], 0,5, color=[0.9,0.9,0.9])
text(-22,2.8,'linearly')
text(-22,2.2,'unstable')
text(-22,1.6,'region')
xlabel('angular frequency Re($z$)')
ylabel('damping $-$Im($z$)')
ylim([-3.5,5])
legend(loc=1)



axB = panel_factory.new_panel(1, 0, r'B')
ts = np.arange(-tmax, tmax+tbin, tbin)

corr_ref, corr_vreeswijk, corr_helias, rate_ref, rate_vreeswijk, rate_helias = get_corr(simtime, N_rec, tmax, tbin, datapath, suffix)

plot(ts, corr_ref, linewidth=2.5, color=mycols[0])
plot(ts, corr_vreeswijk, linewidth=1., color=mycols[1])
plot(ts, corr_helias, linewidth=1., color=mycols[2])

xlim([-20,20])
ylim([-1.1,2.4])
xlabel(r'time lag $\Delta \; (\mathrm{ms})$')
ylabel(r'covariance $\frac{c}{r^2}$')

savefig(figurepath + 'fig2.eps')

