# Sacha van Albada, Nov 2014

# Plot of data created with binary_network_asymm.py

import numpy
import os
from plotfuncs import *
from ginzburg_theory import ginzburg

binarydatapath = '../data/binary/asymm/mcculloch_pitts/'
figurepath = '../figures/'

N_full = 5000
dt_samp = 0.3 # ms sampling resolution
frac_recorded = 0.5

# downscaling factors for helias and vreeswijk scaling
scaling = 0.75
extreme_scaling = 0.3

mycols = [['#1d44b8', '#4169e1', '#839deb'], # blue from dark to light
          ['#6533cb', '#9370db', '#c1adea'], # purple
          ['#af143c', '#c75a76', '#dfa1b1']] # pinkish red

def loaddata(dp):
    files = os.listdir(dp)
    exc1_file = [files[n] for n in xrange(len(files)) if files[n].find('exc1asymm')!=-1]
    exc2_file = [files[n] for n in xrange(len(files)) if files[n].find('exc2asymm')!=-1]
    inh1_file = [files[n] for n in xrange(len(files)) if files[n].find('inh1asymm')!=-1]
    inh2_file = [files[n] for n in xrange(len(files)) if files[n].find('inh2asymm')!=-1]
    print 'datapath: ' + dp
    print 'exc1_file: ' + exc1_file[0]
    print 'exc2_file: ' + exc2_file[0]
    print 'inh1_file: ' + inh1_file[0]
    print 'inh2_file: ' + inh2_file[0]
    m_exc1 = numpy.loadtxt(dp + exc1_file[0])
    m_exc2 = numpy.loadtxt(dp + exc2_file[0])
    m_inh1 = numpy.loadtxt(dp + inh1_file[0])
    m_inh2 = numpy.loadtxt(dp + inh2_file[0])  
    return m_exc1, m_exc2, m_inh1, m_inh2


def compute_corr(m_exc1, m_exc2, m_inh1, m_inh2, Nrec):
    mean_exc1 = numpy.mean(m_exc1)
    mean_inh1 = numpy.mean(m_inh1)
    mean_exc2 = numpy.mean(m_exc2)
    mean_inh2 = numpy.mean(m_inh2)
    cc_ee = 1./(len(m_exc1)*Nrec**2) * numpy.correlate(m_exc1-mean_exc1, m_exc2-mean_exc2, mode='full')
    cc_ei = 1./(len(m_exc1)*Nrec**2) * numpy.correlate(m_exc1-mean_exc1, m_inh1-mean_inh1, mode='full')
    cc_ii = 1./(len(m_exc1)*Nrec**2) * numpy.correlate(m_inh1-mean_inh1, m_inh2-mean_inh2, mode='full')
    return cc_ee,  cc_ei,  cc_ii


def theory_corr(input_type, N, J_array, K_array, m_x, mu_x, sigma_x, theta, tau_m, tmin, tmax, dt):
    tlag = np.array([tmin, tmax, dt])
    nlag = (tmax-tmin)/dt + 1
    # starting guess for mean activities - this is important, as the results can depend on this
    m0 = [0.17,0.08]
    network = ginzburg(numpy.array([N,N,N]), J_array, K_array, m_x, theta)
    try:
        ms, cs = network.solve_self_consistently_corr(m0, mu_x, sigma_x)
        print 'ms: ', ms
        print 'cs: ', cs
    except:
        print 'could not determine theoretical mean activities and zero-lag correlations'
    
    try:
        c_theory = network.corr_vs_lag(m0, tau_m, tlag, mu_x, sigma_x)
    except:
        print 'could not compute c_theory'
        c_theory = np.zeros([2,2,nlag])*np.nan
    return c_theory


################### Load data ####################
# full-scale
m_exc1_full, m_exc2_full, m_inh1_full, m_inh2_full = loaddata(binarydatapath + 'full_scale/')

print 'full-scale rates: '
print 'E: ', mean([m_exc1_full, m_exc2_full])/(N_full*frac_recorded)
print 'I: ', mean([m_inh1_full, m_inh2_full])/(N_full*frac_recorded)

# naive scaling of N and K by the same factor; J propto 1/K; no adjustment of external drive
m_exc1_naive, m_exc2_naive, m_inh1_naive, m_inh2_naive = loaddata(binarydatapath + 'naive_scaling/')

print 'rates after naive scaling: '
print 'E: ', mean([m_exc1_naive, m_exc2_naive])/(N_full*frac_recorded*0.5)
print 'I: ', mean([m_inh1_naive, m_inh2_naive])/(N_full*frac_recorded*0.5)

# scaling N and K by the same factor; J propto 1/K
m_exc1_helias, m_exc2_helias, m_inh1_helias, m_inh2_helias = loaddata(binarydatapath + 'helias_scaling/')
print 'rates after helias scaling: '
print 'E: ', mean([m_exc1_helias, m_exc2_helias])/(N_full*frac_recorded*scaling)
print 'I: ', mean([m_inh1_helias, m_inh2_helias])/(N_full*frac_recorded*scaling)

# scaling N and K by the same factor; J propto 1/sqrt(K)
m_exc1_vreeswijk, m_exc2_vreeswijk, m_inh1_vreeswijk, m_inh2_vreeswijk = loaddata(binarydatapath + 'vreeswijk_scaling/')
print 'rates after vreeswijk scaling: '
print 'E: ', mean([m_exc1_vreeswijk, m_exc2_vreeswijk])/(N_full*frac_recorded*scaling)
print 'I: ', mean([m_inh1_vreeswijk, m_inh2_vreeswijk])/(N_full*frac_recorded*scaling)

# scaling N much more than K; J propto 1/K
m_exc1_extreme_helias, m_exc2_extreme_helias, m_inh1_extreme_helias, m_inh2_extreme_helias = loaddata(binarydatapath + 'extreme_helias/')
print 'rates after extreme helias scaling: '
print 'E: ', mean([m_exc1_extreme_helias, m_exc2_extreme_helias])/(N_full*frac_recorded*extreme_scaling)
print 'I: ', mean([m_inh1_extreme_helias, m_inh2_extreme_helias])/(N_full*frac_recorded*extreme_scaling)

# scaling N much more than K; J propto 1/sqrt(K)
m_exc1_extreme_vreeswijk, m_exc2_extreme_vreeswijk, m_inh1_extreme_vreeswijk, m_inh2_extreme_vreeswijk = loaddata(binarydatapath + 'extreme_vreeswijk/')
print 'rates after extreme vreeswijk scaling: '
print 'E: ', mean([m_exc1_extreme_vreeswijk, m_exc2_extreme_vreeswijk])/(N_full*frac_recorded*extreme_scaling)
print 'I: ', mean([m_inh1_extreme_vreeswijk, m_inh2_extreme_vreeswijk])/(N_full*frac_recorded*extreme_scaling)

################### Compute correlations ####################
print 'computing correlations'

# full-scale
cc_ee_full, cc_ei_full, cc_ii_full = compute_corr(m_exc1_full, m_exc2_full, m_inh1_full, m_inh2_full, int(0.5*N_full))

# naive scaling of N and K by the same factor; J propto 1/K; no adjustment of external drive
cc_ee_naive, cc_ei_naive, cc_ii_naive = compute_corr(m_exc1_naive, m_exc2_naive, m_inh1_naive, m_inh2_naive, int(frac_recorded*N_full*0.5))

# scaling N and K by the same factor; J propto 1/K
cc_ee_helias, cc_ei_helias, cc_ii_helias = compute_corr(m_exc1_helias, m_exc2_helias, m_inh1_helias, m_inh2_helias, int(frac_recorded*N_full*scaling))

# scaling N and K by the same factor; J propto 1/sqrt(K)
cc_ee_vreeswijk, cc_ei_vreeswijk, cc_ii_vreeswijk = compute_corr(m_exc1_vreeswijk, m_exc2_vreeswijk, m_inh1_vreeswijk, m_inh2_vreeswijk, int(frac_recorded*N_full*scaling))

# scaling N much more than K; J propto 1/K
cc_ee_extreme_helias, cc_ei_extreme_helias, cc_ii_extreme_helias = compute_corr(m_exc1_extreme_helias, m_exc2_extreme_helias, m_inh1_extreme_helias, m_inh2_extreme_helias, int(frac_recorded*N_full*extreme_scaling))

# scaling N much more than K; J propto 1/sqrt(K)
cc_ee_extreme_vreeswijk, cc_ei_extreme_vreeswijk, cc_ii_extreme_vreeswijk = compute_corr(m_exc1_extreme_vreeswijk, m_exc2_extreme_vreeswijk, m_inh1_extreme_vreeswijk, m_inh2_extreme_vreeswijk, int(frac_recorded*N_full*extreme_scaling))


################### Theory ####################

tmin_th = -100. # ms
tmax_th = 100. # ms
dt_th = 0.1 # ms
n_th = (tmax_th-tmin_th)/dt_th + 1

# full scale
execfile(binarydatapath + 'full_scale/parameters_binary_asymm.py')
g = ginzburg(np.array([N,N,N]), J_array, K_array, m_x, theta)
ms, cs = g.solve_self_consistently_corr(np.array([0.17, 0.08]), mu_x, sigma_x)
S = g.susceptibility_corr(ms, cs, mu_x, sigma_x)
print 'susceptibility full-scale: ', S
c_theory_full = theory_corr(input_type, N, J_array, K_array, m_x, mu_x, sigma_x, theta, tau_m, tmin_th, tmax_th, dt_th)

# scaling N and K by the same factor; J propto 1/K
execfile(binarydatapath + 'helias_scaling/parameters_binary_asymm.py')
g = ginzburg(np.array([N,N,N]), J_array, K_array, m_x, theta)
ms, cs = g.solve_self_consistently_corr(np.array([0.17, 0.08]), mu_x, sigma_x)
S = g.susceptibility_corr(ms, cs, mu_x, sigma_x)
print 'susceptibility helias: ', S

# scaling N and K by the same factor; J propto 1/sqrt(K)
execfile(binarydatapath + 'vreeswijk_scaling/parameters_binary_asymm.py')
g = ginzburg(np.array([N,N,N]), J_array, K_array, m_x, theta)
ms, cs = g.solve_self_consistently_corr(np.array([0.17, 0.08]), mu_x, sigma_x)
S = g.susceptibility_corr(ms, cs, mu_x, sigma_x)
print 'susceptibility vreeswijk: ', S

################### Plot ####################

tlag = numpy.arange(len(cc_ei_full))*dt_samp - len(cc_ei_full)/2*dt_samp
tlag_th = np.arange(tmin_th, tmax_th+dt_th, dt_th)

xmin = -30. # ms
xmax = 60.

scale = 1.
width = 6.83 
n_horz_panels = 2
n_vert_panels = 2

panel_factory = create_fig_plos(1, scale, width, n_horz_panels, n_vert_panels)

f = 1e5

axA = panel_factory.new_panel(0, 0, r'A')

plot(tlag, f*cc_ee_full, color=mycols[0][0], linewidth=2, label=r'$c_{\mathrm{ee}}$')
plot(tlag, f*cc_ei_full, color=mycols[1][0], linewidth=2, label=r'$c_{\mathrm{ei}}$')
plot(tlag, f*cc_ii_full, color=mycols[2][0], linewidth=2, label=r'$c_{\mathrm{ii}}$')

plot(tlag_th, f*c_theory_full[0,0], '--', linewidth=2, color=mycols[0][0])
plot(tlag_th, f*c_theory_full[0,1], '--', linewidth=2, color=mycols[1][0])
plot(tlag_th, f*c_theory_full[1,1], '--', linewidth=2, color=mycols[2][0])

rcParams['legend.fontsize'] = scale*10

xlim([xmin,xmax])
xlabel(r'$\Delta$' + ' (ms)')
ylabel(r'$c \; (10^{-5})$')
title('full scale')
legend(loc=4)

# naive scaling
axB = panel_factory.new_panel(1, 0, r'B')

plot(tlag, f*cc_ee_full, linewidth=2, color=mycols[0][0], label='full')
plot(tlag, f*cc_ei_full, linewidth=2, color=mycols[1][0])
plot(tlag, f*cc_ii_full, linewidth=2, color=mycols[2][0])

plot(tlag, f*cc_ee_naive*0.5, linewidth=1, color=mycols[0][1], label=r'$\frac{N}{N_0}\!=\!0.75; \frac{K}{K_0}\!=\!0.75$')
plot(tlag, f*cc_ei_naive*0.5, linewidth=1, color=mycols[1][1])
plot(tlag, f*cc_ii_naive*0.5, linewidth=1, color=mycols[2][1])

xlim([xmin,xmax])
xlabel(r'$\Delta$' + ' (ms)')
ylabel(r'$\frac{N}{N_0} c \; (10^{-5})$')
title(r'$J \propto 1/K$' + ', fixed drive')

rcParams['legend.fontsize'] = scale*6
legend(loc=4)

# helias scaling
axC = panel_factory.new_panel(0, 1, r'C')

plot(tlag, f*cc_ee_full, linewidth=2, color=mycols[0][0], label='full')
plot(tlag, f*cc_ei_full, linewidth=2, color=mycols[1][0])
plot(tlag, f*cc_ii_full, linewidth=2, color=mycols[2][0])

plot(tlag, f*cc_ee_helias*scaling, linewidth=1, color=mycols[0][1], label=r'$\frac{N}{N_0}\!=\!0.75; \frac{K}{K_0}\!=\!0.75$')
plot(tlag, f*cc_ei_helias*scaling, linewidth=1, color=mycols[1][1])
plot(tlag, f*cc_ii_helias*scaling, linewidth=1, color=mycols[2][1])

plot(tlag, f*cc_ee_extreme_helias*extreme_scaling, linewidth=0.5, color=mycols[0][2], label=r'$\frac{N}{N_0}\!=\!0.3; \frac{K}{K_0}\!=\!0.75$')
plot(tlag, f*cc_ei_extreme_helias*extreme_scaling, linewidth=0.5, color=mycols[1][2])
plot(tlag, f*cc_ii_extreme_helias*extreme_scaling, linewidth=0.5, color=mycols[2][2])

xlim([xmin,xmax])
xlabel(r'$\Delta$' + ' (ms)')
ylabel(r'$\frac{N}{N_0} c \; (10^{-5})$')
title(r'$J \propto 1/K$' + ', adjusting drive')
legend(loc=4)

# vreeswijk scaling
axD = panel_factory.new_panel(1, 1, r'D')

plot(tlag, f*cc_ee_full, linewidth=2, color=mycols[0][0], label='full')
plot(tlag, f*cc_ei_full, linewidth=2, color=mycols[1][0])
plot(tlag, f*cc_ii_full, linewidth=2, color=mycols[2][0])

plot(tlag, f*cc_ee_vreeswijk*scaling, linewidth=1, color=mycols[0][1], label=r'$\frac{N}{N_0}\!=\!0.75; \frac{K}{K_0}\!=\!0.75$')
plot(tlag, f*cc_ei_vreeswijk*scaling, linewidth=1, color=mycols[1][1])
plot(tlag, f*cc_ii_vreeswijk*scaling, linewidth=1, color=mycols[2][1])

plot(tlag, f*cc_ee_extreme_vreeswijk*extreme_scaling, linewidth=0.5, color=mycols[0][2], label=r'$\frac{N}{N_0}\!=\!0.3; \frac{K}{K_0}\!=\!0.75$')
plot(tlag, f*cc_ei_extreme_vreeswijk*extreme_scaling, linewidth=0.5, color=mycols[1][2])
plot(tlag, f*cc_ii_extreme_vreeswijk*extreme_scaling, linewidth=0.5, color=mycols[2][2])

xlim([xmin,xmax])
xlabel(r'$\Delta$' + ' (ms)')
ylabel(r'$\frac{N}{N_0} c \; (10^{-5})$')
title(r'$J \propto 1/\sqrt{K}$' + ', adjusting drive')
legend(loc=4)

savefig(figurepath + 'fig3.eps')
