# Sacha van Albada, April 2015

# Script for plotting correlations from theory and simulations
# for an E-I LIF network with different firing rates

# Data created using hambach_run_LIF_network_rate_comparison.py

import siegert as sig
import numpy
from plotfuncs import *
from pylab import *
import hawkesnet as hn
from corr_hawkes_like import *
from ps_manip import *
from parameters_LIF_network_rate_comparison import *
import scipy.optimize as sciopt

figurepath = '../figures/'
datapath = '../data/spiking/LIF_network_rate_comparison/'

mycols = [black, '#1d771d', '#a8d57b']

procs = 24
N_rec_full = 1000
N_rec_acf = 100
g = 5.0
gamma = 0.25
epsilon = 0.1
NE_full = 8000

mu_ext_full = [10., 25., 10., 25.]
sigma_ext_full = [5., 20., 5., 20.]
K_scaling = [0.5, 0.5, 0.5, 0.5]
N_scaling = [1, 1, 0.5, 0.5]
J = 0.1

N_rec_scaled = numpy.array(N_scaling)*N_rec_full

delay = 3.0

mu = [8.68, 13.16, 8.68, 13.16]
sigma = [5.37, 25.68, 5.37, 25.68]

# number of neurons in simulations determining impulse response
N = 1000
# simulation time of impulse response simulations
T = 50000.0 # ms
# delay in impulse response simulations (sim_resp_sigma_dep_fw.py)
d = 0.1
# histogram range/resolution
tau_max = 50.0
delta_tau = 0.1

ts_impulse = arange(-tau_max, tau_max+delta_tau, delta_tau)

which_corr = 'ei' # which to plot: 'ee', 'ei', or 'ii'

####### Determine correlations from theory and simulations #######

def get_auto_data(filename, T, tmax, tbin):
    """Get normalized correlation data"""
    
    corr_data = numpy.loadtxt(filename)
    corr = corr_data[2:]
    count1 = corr_data[0]
    count2 = corr_data[1]
    T_eff = T - tmax    # used new feature of correlation detector to start recording after tmax to avoid triangular falloff to the sides
    assert(count1 == count2)
    rate = count1 / (T_eff/1000.0)
    corr = 1.0*corr/T_eff*1000.0/tbin*1000.0

    return corr, rate


def get_auto(T, N_rec, tmax, tbin, datapath, suffix, n_proc):
    """simulated autocorrelations"""

    auto_ee = numpy.zeros(2*tmax/tbin+1)
    rate_e = 0.
    auto_ii = numpy.zeros(2*tmax/tbin+1)
    rate_i = 0.
    ce = ci = 0

    for p in range(n_proc):
        auto_ee_p, rate_e_p = get_auto_data(datapath + 'acf_ee'+ suffix + str(p), T, tmax, tbin)
        
        if rate_e_p > 0.0:
            auto_ee += auto_ee_p
            rate_e += rate_e_p
            ce += 1

        auto_ii_p, rate_i_p = get_auto_data(datapath + 'acf_ii' + suffix + str(p), T, tmax, tbin)
        
        if rate_i_p > 0.0:
            auto_ii += auto_ii_p
            rate_i += rate_i_p
            ci += 1
                   
    return auto_ee*(1./N_rec_acf), auto_ii*(1./N_rec_acf), rate_e*(1./N_rec_acf), rate_i*(1./N_rec_acf)


def analyse_one_w(filename, N, T, d, sigma, delta_tau, w_c):

    raw_data = loadtxt(filename)
    n_pre = raw_data[0] # number presynaptic events
    n_post = raw_data[1] # number postsynaptic events of whole population
    hist_counts = raw_data[2:] # raw histogram counts of correlometer
      
    T_eff = T - tau_max    # used new feature of correlation detector to start recording after tmax to avoid triangular falloff to the sides
    n_expected = n_post*n_pre*delta_tau/T_eff # expected count per bin
    print "means:", n_expected, mean(hist_counts[:len(hist_counts)/2-10]) # compare to baseline
    
    rate = n_post / (N*T_eff/1000.0)
    print 'rate (impulse response simulation) = ', rate
    
    int_resp = sum(hist_counts - n_expected) * 1.0/(N*n_pre)
    
    hist_norm = hist_counts * 1.0/(N*n_pre * delta_tau/1000.0)
  
    # index of bin at zero time lag
    i_t0 = int((tau_max + d)/delta_tau)
    nu_inst0 = hist_norm[i_t0]

    return rate, hist_norm, int_resp, nu_inst0


def fit_time_constant(w_c, mu, sigma):

  wcstr = str(round(w_c*100+0.001)/100.0)
  filename = datapath + 'response_J'+str(J)+'_Jc'+ wcstr +'mu'+str(mu)+'sigma'+str(sigma)+'.dat'

  print filename

  rate,\
  hist_norm,\
  int_sim,\
  nu_inst_sim = analyse_one_w(filename, N, T, d, sigma, delta_tau, w_c)

  int_ana_plus = sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, w_c)[0]
  
  # determine effective time constant from fit 
  def err(tau_e, ts_impulse, rs):
    return int_ana_plus/tau_e*np.exp(-ts_impulse / tau_e) * (ts_impulse > 0) * 1000 - rs 

  res = sciopt.leastsq(err, 4.0, args=(ts_impulse, hist_norm-rate))
  print "result of fit =", res

  return res[0][0], hist_norm


def get_corr_data(filename, n1, n2, T, tmax, tbin):
    """Get normalized correlation data"""
    
    corr_data = numpy.loadtxt(filename)
    corr = corr_data[2:]
    count1 = corr_data[0]
    count2 = corr_data[1]
    T_eff = T - tmax 
    rate1 = count1 / (n1*T_eff/1000.0)
    rate2 = count2 / (n2*T_eff/1000.0)
    rate = numpy.sqrt(rate1*rate2)
    corr = 1.0*corr/(n1*n2)/T_eff*1000.0/tbin*1000.0 / (rate1*rate2) - 1.

    return corr, rate


def get_corr(T, N_rec, tmax, tbin, datapath, suffix):
    """Plot simulated correlations"""

    corr_ee, rate_e = get_corr_data(datapath + 'corr_ee' + suffix, N_rec, N_rec, T, tmax, tbin)
    corr_ei, rate_ei = get_corr_data(datapath + 'corr_ei' + suffix, N_rec, N_rec, T, tmax, tbin)
    corr_ii, rate_i = get_corr_data(datapath + 'corr_ii' + suffix, N_rec, N_rec, T, tmax, tbin)
    corr_mm, rate_m = get_corr_data(datapath + 'corr_pp' + suffix, N_rec, N_rec, T, tmax, tbin)

    return corr_ee, corr_ii, corr_ei, corr_mm, rate_e, rate_i, rate_m


def corr_theory_hawkes(nu, mu, sigma, g, J, tau_e, delay, NE, NI, epsilon):

    print 'mu: ', mu
    print 'sigma: ', sigma

    w = sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, J)[0]
    gw = sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, -g*J)[0]

    print 'effective weights = ', w, gw
    print 'delay = ', delay

    # The penultimate parameter of hawkesnet is a rate parameter.
    # It doesn't matter for computing C, because the rate given
    # as an argument to cross_cov_matrix2 is used.
    h_ana = hn.hawkesnet(NE, NI, epsilon, g, w, tau_e, delay, 0., 30)
    C = h_ana.cross_cov_matrix2(w, g, nu)
    Cs = [C(t) for t in ts_ana]

    C_ee = numpy.array([C[0,0]/(nu)**2 for C in Cs])
    C_ei = numpy.array([C[0,1]/(nu)**2 for C in Cs])
    C_ie = numpy.array([C[1,0]/(nu)**2 for C in Cs])
    C_ii = numpy.array([C[1,1]/(nu)**2 for C in Cs])

    return C_ee, C_ei, C_ie, C_ii

ts = numpy.arange(-tmax, tmax+tbin, tbin)

ts_ana = arange(-50., 50., 0.1)

rates_e = []
rates_i = []
rates_e_vreeswijk = []
rates_i_vreeswijk = []
rates_e_helias = []
rates_i_helias = []

rates_ea = []
rates_ia = []
rates_ea_vreeswijk = []
rates_ia_vreeswijk = []
rates_ea_helias = []
rates_ia_helias = []

sigma_a_E = numpy.zeros(len(mu_ext_full)+1)
sigma_a_I = numpy.zeros(len(mu_ext_full)+1)

auto_ees = []
auto_iis = []
auto_ees_vreeswijk = []
auto_iis_vreeswijk = []
auto_ees_helias = []
auto_iis_helias = []

corr_ees = []
corr_iis = []
corr_eis = []
corr_ees_vreeswijk = []
corr_iis_vreeswijk = []
corr_eis_vreeswijk = []
corr_ees_helias = []
corr_iis_helias = []
corr_eis_helias = []

C_ee = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ei = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ie = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ii = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))

C_ee_helias = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ei_helias = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ie_helias = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ii_helias = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))

C_ee_vreeswijk = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ei_vreeswijk = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ie_vreeswijk = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))
C_ii_vreeswijk = numpy.zeros((len(ts_ana), len(mu_ext_full)+1))

impulse_responses = []

# Parameters for analysis of poles (the first few parameters have dummy values
# for initializing the corr_hawkes_like object)
w = 1.
nu = 1.
alpha = 1.
poles_max = 5.
branch_max = 5

for i in xrange(len(mu_ext_full)):
    suffix = 'none' + 'K_scaling1' + 'NE' + str(NE_full) + 'J' + str(J) + 'delay' + str(delay) + 'mu_ext_full' + str(mu_ext_full[i]) + 'sigma_ext_full' + str(sigma_ext_full[i]) + '.dat'
    print suffix

    # cross-corr
    corr_ee, corr_ii, corr_ei, corr_mm, rate_e, rate_i, rate_m = get_corr(simtime, N_rec_full, tmax, tbin, datapath, suffix)

    # auto corr
    auto_ee, auto_ii, rate_ea, rate_ia = get_auto(simtime, N_rec_acf, tmax, tbin, datapath, suffix, procs)

    print 'rate_ea: ', rate_ea
    print 'rate_ia: ', rate_ia

    int_auto_E = sum(auto_ee-rate_ea**2)*tbin
    int_auto_I = sum(auto_ii-rate_ia**2)*tbin
  
    # sigma_a correlation hole - needed to plug into parameters_response_auto_high_fb.py
    sigma_a_E[i] = (rate_ea*1000. - int_auto_E) / (rate_ea**2 * sqrt(2.*pi))
    sigma_a_I[i] = (rate_ia*1000. - int_auto_I) / (rate_ia**2 * sqrt(2.*pi))

    print 'i: ', i
    print 'rate_e: ', rate_e
    print 'rate_i: ', rate_i
    print 'sigma_a_E: ', sigma_a_E[i]
    print 'sigma_a_I: ', sigma_a_I[i]
    rates_e.append(rate_e)
    rates_i.append(rate_i)
    rates_ea.append(rate_ea)
    rates_ia.append(rate_ia)
    auto_ees.append(auto_ee)
    auto_iis.append(auto_ii)
    corr_ees.append(corr_ee)
    corr_iis.append(corr_ii)
    corr_eis.append(corr_ei)

    # the first argument of fit_time_constant indicates the synaptic strength
    # used in the simulations for estimating the impulse response
    tau_e, impulse_response = fit_time_constant(1.0, mu[i], sigma[i])
    print 'tau_e: ', tau_e
    impulse_responses.append(impulse_response)

    C_ee[:,i], C_ei[:,i], C_ie[:,i], C_ii[:,i] = corr_theory_hawkes(rates_e[i], mu[i], sigma[i], g, J, tau_e, delay, NE_full, round(gamma*NE_full), epsilon)

    # load data from scaled networks
    suffix = 'vreeswijk' + 'K_scaling' + str(K_scaling[i]) + 'NE' + str(int(N_scaling[i]*NE_full)) + 'J' + str(J) + 'delay' + str(delay) + 'mu_ext_full' + str(mu_ext_full[i]) + 'sigma_ext_full' + str(sigma_ext_full[i]) + '.dat'
    print suffix
    # cross-corr
    corr_ee, corr_ii, corr_ei, corr_mm, rate_e, rate_i, rate_m = get_corr(simtime, N_rec_scaled[i], tmax, tbin, datapath, suffix)

    # auto corr
    auto_ee, auto_ii, rate_ea, rate_ia = get_auto(simtime, N_rec_acf, tmax, tbin, datapath, suffix, procs)

    print 'rate_e: ', rate_e
    print 'rate_i: ', rate_i

    rates_e_vreeswijk.append(rate_e)
    rates_i_vreeswijk.append(rate_i)
    rates_ea_vreeswijk.append(rate_ea)
    rates_ia_vreeswijk.append(rate_ia)
    auto_ees_vreeswijk.append(auto_ee)
    auto_iis_vreeswijk.append(auto_ii)
    corr_ees_vreeswijk.append(corr_ee)
    corr_iis_vreeswijk.append(corr_ii)
    corr_eis_vreeswijk.append(corr_ei)

    C_ee_vreeswijk[:,i], C_ei_vreeswijk[:,i], C_ie_vreeswijk[:,i], C_ii_vreeswijk[:,i] = corr_theory_hawkes(rates_e_vreeswijk[i], mu[i], sigma[i], g, J/sqrt(K_scaling[i]), tau_e, delay, N_scaling[i]*NE_full, round(gamma*N_scaling[i]*NE_full), epsilon*K_scaling[i]/N_scaling[i])

    suffix = 'helias' + 'K_scaling' + str(K_scaling[i]) + 'NE' + str(int(N_scaling[i]*NE_full)) + 'J' + str(J) + 'delay' + str(delay) + 'mu_ext_full' + str(mu_ext_full[i]) + 'sigma_ext_full' + str(sigma_ext_full[i]) + '.dat'
    print suffix

    # cross-corr
    corr_ee, corr_ii, corr_ei, corr_mm, rate_e, rate_i, rate_m = get_corr(simtime, N_rec_scaled[i], tmax, tbin, datapath, suffix)

    # auto corr
    auto_ee, auto_ii, rate_ea, rate_ia = get_auto(simtime, N_rec_acf, tmax, tbin, datapath, suffix, procs)

    print 'rate_e: ', rate_e
    print 'rate_i: ', rate_i

    rates_e_helias.append(rate_e)
    rates_i_helias.append(rate_i)
    rates_ea_helias.append(rate_ea)
    rates_ia_helias.append(rate_ia)
    auto_ees_helias.append(auto_ee)
    auto_iis_helias.append(auto_ii)
    corr_ees_helias.append(corr_ee)
    corr_iis_helias.append(corr_ii)
    corr_eis_helias.append(corr_ei)

    C_ee_helias[:,i], C_ei_helias[:,i], C_ie_helias[:,i], C_ii_helias[:,i] = corr_theory_hawkes(rates_e_helias[i], mu[i], sigma[i], g, J/K_scaling[i], tau_e, delay, N_scaling[i]*NE_full, round(gamma*N_scaling[i]*NE_full), epsilon*K_scaling[i]/N_scaling[i])

####### Plot #######

scale = 1.0
width = 6.83 # inches for PloS double column
n_horz_panels = 2
n_vert_panels = 2

f = 100. # scaling of y axis
ylab1 = r'$\frac{c}{r^2} (10^{-2})$'
ylab2 = r'$\frac{c}{r^2} \frac{N}{N_0} (10^{-2})$'

xmin = -30.
xmax = 30.
xtcks = [-20., -10., 0., 10., 20.]

xmin_auto = -30
xmax_auto = 30
ymin_auto = -1.5
ymax_auto = 4.0

lw_full = 0.5 
lw_scaled = 0.5 
lw_ana = 1.5 

panel_factory = create_fig_plos(1, scale, width, n_horz_panels, n_vert_panels)

axA = panel_factory.new_panel(0, 0, r'A')

rcParams['axes.titlesize'] = scale*8
title(r'$\nu_{\mathrm{full}} =' + str(round(rates_e[0],1)) + '$' + ' spikes/s, ' + r'$N=N_0$')

if which_corr == 'ee':
    corr_full = corr_ees[0]
    corr_vreeswijk = corr_ees_vreeswijk[0]
    corr_helias = corr_ees_helias[0]
    corr_ana = C_ee[:,0]
if which_corr == 'ei':
    corr_full = corr_eis[0][-1::-1]
    corr_vreeswijk = corr_eis_vreeswijk[0][-1::-1]
    corr_helias = corr_eis_helias[0][-1::-1]
    corr_ana = C_ei[:,0]
if which_corr == 'ii':
    corr_full = corr_iis[0]
    corr_vreeswijk = corr_iis_vreeswijk[0]
    corr_helias = corr_iis_helias[0]
    corr_ana = C_ii[:,0]

plot(ts, f*corr_full, linewidth=lw_full, color=mycols[0])
plot(ts, f*corr_vreeswijk*N_scaling[0], linewidth=lw_scaled, color=mycols[1])
plot(ts, f*corr_helias*N_scaling[0], linewidth=lw_scaled, color=mycols[2])
plot(ts_ana, f*corr_ana, '--', color='k', linewidth=lw_ana)

rcParams['legend.fontsize'] = scale*10
legend()

xlim([xmin, xmax])
ylim([-3, 7])
xticks(xtcks)
xlabel(r'$\Delta \; (\mathrm{ms})$')
ylabel(ylab1)

a = axes([.12, .815, .09, .09])
plot(ts, auto_ees[0]/rates_ea[0]**2-1., color='k')
plot(ts, auto_ees_vreeswijk[0]/rates_ea_vreeswijk[0]**2-1, color=mycols[1])
plot(ts, auto_ees_helias[0]/rates_ea_helias[0]**2-1, color=mycols[2])
xlim([xmin_auto, xmax_auto])
ylim([ymin_auto, ymax_auto])
setp(a, xticks=[], yticks=[])

axB = panel_factory.new_panel(1, 0, r'B')

title(r'$\nu_{\mathrm{full}} =' + str(round(rates_e[1],1)) + '$' + ' spikes/s, ' + r'$N=N_0$')

if which_corr == 'ee':
    corr_full = corr_ees[1]
    corr_vreeswijk = corr_ees_vreeswijk[1]
    corr_helias = corr_ees_helias[1]
    corr_ana = C_ee[:,1]
if which_corr == 'ei':
    corr_full = corr_eis[1][-1::-1]
    corr_vreeswijk = corr_eis_vreeswijk[1][-1::-1]
    corr_helias = corr_eis_helias[1][-1::-1]
    corr_ana = C_ei[:,1]
if which_corr == 'ii':
    corr_full = corr_iis[1]
    corr_vreeswijk = corr_iis_vreeswijk[1]
    corr_helias = corr_iis_helias[1]
    corr_ana = C_ii[:,1]

plot(ts, f*corr_full, linewidth=lw_full, color=mycols[0])
plot(ts, f*corr_vreeswijk*N_scaling[1], linewidth=lw_scaled, color=mycols[1])
plot(ts, f*corr_helias*N_scaling[1], linewidth=lw_scaled, color=mycols[2])
plot(ts_ana, f*corr_ana, '--', color='k', linewidth=lw_ana)

rcParams['legend.fontsize'] = scale*10
legend()

xlim([xmin, xmax])
ylim([-1, 1.7])
xticks(xtcks)
yticks([-1,0,1])
xlabel(r'$\Delta \; (\mathrm{ms})$')
ylabel(ylab1)

a = axes([.62, .815, .09, .09])
plot(ts, auto_ees[1]/rates_ea[1]**2-1., color='k')
plot(ts, auto_ees_vreeswijk[1]/rates_ea_vreeswijk[1]**2-1, color=mycols[1])
plot(ts, auto_ees_helias[1]/rates_ea_helias[1]**2-1, color=mycols[2])
xlim([xmin_auto, xmax_auto])
ylim([ymin_auto, ymax_auto])
setp(a, xticks=[], yticks=[])

axC = panel_factory.new_panel(0, 1, r'C')

title(r'$\nu_{\mathrm{full}} =' + str(round(rates_e[2],1)) + '$' + ' spikes/s, ' + r'$N\propto K$')

if which_corr == 'ee':
    corr_full = corr_ees[2]
    corr_vreeswijk = corr_ees_vreeswijk[2]
    corr_helias = corr_ees_helias[2]
    corr_ana = C_ee[:,2]
if which_corr == 'ei':
    corr_full = corr_eis[2][-1::-1]
    corr_vreeswijk = corr_eis_vreeswijk[2][-1::-1]
    corr_helias = corr_eis_helias[2][-1::-1]
    corr_ana = C_ei[:,2]
if which_corr == 'ii':
    corr_full = corr_iis[2]
    corr_vreeswijk = corr_iis_vreeswijk[2]
    corr_helias = corr_iis_helias[2]
    corr_ana = C_ii[:,2]

plot(ts, f*corr_full, linewidth=lw_full, color=mycols[0])
plot(ts, f*corr_vreeswijk*N_scaling[2], linewidth=lw_scaled, color=mycols[1])
plot(ts, f*corr_helias*N_scaling[2], linewidth=lw_scaled, color=mycols[2])
plot(ts_ana, f*corr_ana, '--', color='k', linewidth=lw_ana)

rcParams['legend.fontsize'] = scale*10
legend()

xlim([xmin, xmax])
ylim([-3, 7])
xticks(xtcks)
xlabel(r'$\Delta \; (\mathrm{ms})$')
ylabel(ylab2)

a = axes([.12, .34, .09, .09])
plot(ts, auto_ees[2]/rates_ea[2]**2-1., color='k')
plot(ts, auto_ees_vreeswijk[2]/rates_ea_vreeswijk[2]**2-1, color=mycols[1])
plot(ts, auto_ees_helias[2]/rates_ea_helias[2]**2-1, color=mycols[2])
xlim([xmin_auto, xmax_auto])
ylim([ymin_auto, ymax_auto])
setp(a, xticks=[], yticks=[])

axD = panel_factory.new_panel(1, 1, r'D')

title(r'$\nu_{\mathrm{full}} =' + str(round(rates_e[3],1)) + '$' + ' spikes/s, ' + r'$N\propto K$')

if which_corr == 'ee':
    corr_full = corr_ees[3]
    corr_vreeswijk = corr_ees_vreeswijk[3]
    corr_helias = corr_ees_helias[3]
    corr_ana = C_ee[:,3]
if which_corr == 'ei':
    corr_full = corr_eis[3][-1::-1]
    corr_vreeswijk = corr_eis_vreeswijk[3][-1::-1]
    corr_helias = corr_eis_helias[3][-1::-1]
    corr_ana = C_ei[:,3]
if which_corr == 'ii':
    corr_full = corr_iis[3]
    corr_vreeswijk = corr_iis_vreeswijk[3]
    corr_helias = corr_iis_helias[3]
    corr_ana = C_ii[:,3]

plot(ts, f*corr_full, linewidth=lw_full, color=mycols[0], label='full')
plot(ts, f*corr_vreeswijk*N_scaling[3], linewidth=lw_scaled, color=mycols[1], label=r'$J\propto1/\sqrt{K}$')
plot(ts, f*corr_helias*N_scaling[3], linewidth=lw_scaled, color=mycols[2], label=r'$J\propto1/K$')
plot(ts_ana, f*corr_ana, '--', color='k', linewidth=lw_ana)

rcParams['legend.fontsize'] = scale*8
legend()

xlim([xmin, xmax])
ylim([-1., 1.7])
xticks(xtcks)
yticks([-1,0,1])
xlabel(r'$\Delta \; (\mathrm{ms})$')
ylabel(ylab2)

a = axes([.62, .34, .09, .09])
plot(ts, auto_ees[3]/rates_ea[3]**2-1., color='k')
plot(ts, auto_ees_vreeswijk[3]/rates_ea_vreeswijk[3]**2-1, color=mycols[1])
plot(ts, auto_ees_helias[3]/rates_ea_helias[3]**2-1, color=mycols[2])
xlim([xmin_auto, xmax_auto])
ylim([ymin_auto, ymax_auto])
setp(a, xticks=[], yticks=[])

savefig(figurepath + 'fig5.eps')


