# Sacha van Albada, Nov 2014

# Plot of data created with binary_network_symm.py

import numpy
import os
import pylab as pl
from plotfuncs import *
from ps_manip import *
from ginzburg_theory import ginzburg

binarydatapath = '../data/binary/symm/'
figurepath = '../figures/'

execfile(binarydatapath + 'full_scale/parameters_binary_symm.py')
N_rec_full = int(numpy.copy(N_rec))
execfile(binarydatapath + 'single_step_downscaled/parameters_binary_symm.py')
N_rec_1step = int(numpy.copy(N_rec))
execfile(binarydatapath + 'two_step_downscaled/parameters_binary_symm.py')
N_rec_2step = int(numpy.copy(N_rec))
execfile(binarydatapath + 'Vreeswijk_scaling/parameters_binary_symm.py')
N_rec_vv = int(numpy.copy(N_rec))

mycols = [['#1d44b8', '#4169e1', '#839deb'], # blue from dark to light
          ['#6533cb', '#9370db', '#c1adea'], # purple
          ['#af143c', '#c75a76', '#dfa1b1']] # pinkish red

mycols2 = ['#1d771d', '#a8d57b']

def loaddata(dp):
    files = os.listdir(dp)
    exc1_file = [files[n] for n in xrange(len(files)) if files[n].find('exc1symm')!=-1]
    exc2_file = [files[n] for n in xrange(len(files)) if files[n].find('exc2symm')!=-1]
    inh1_file = [files[n] for n in xrange(len(files)) if files[n].find('inh1symm')!=-1]
    inh2_file = [files[n] for n in xrange(len(files)) if files[n].find('inh2symm')!=-1]
    print 'datapath: ' + dp
    print 'exc1_file: ' + exc1_file[0]
    print 'exc2_file: ' + exc2_file[0]
    print 'inh1_file: ' + inh1_file[0]
    print 'inh2_file: ' + inh2_file[0]
    m_exc1 = numpy.loadtxt(dp + exc1_file[0])
    m_exc2 = numpy.loadtxt(dp + exc2_file[0])
    m_inh1 = numpy.loadtxt(dp + inh1_file[0])
    m_inh2 = numpy.loadtxt(dp + inh2_file[0])  
    return m_exc1, m_exc2, m_inh1, m_inh2


def correlate(m_exc1, m_exc2, m_inh1, m_inh2, Nrec):
    mean_exc1 = numpy.mean(m_exc1)
    mean_inh1 = numpy.mean(m_inh1)
    mean_exc2 = numpy.mean(m_exc2)
    mean_inh2 = numpy.mean(m_inh2)
    cc_ee = 1./(len(m_exc1)*Nrec**2) * numpy.correlate(m_exc1-mean_exc1, m_exc2-mean_exc2, mode='full')
    cc_ei = 1./(len(m_exc1)*Nrec**2) * numpy.correlate(m_exc1-mean_exc1, m_inh1-mean_inh1, mode='full')
    cc_ii = 1./(len(m_exc1)*Nrec**2) * numpy.correlate(m_inh1-mean_inh1, m_inh2-mean_inh2, mode='full')
    return cc_ee,  cc_ei,  cc_ii



# full-scale

m_exc1_full, m_exc2_full, m_inh1_full, m_inh2_full = loaddata(binarydatapath + 'full_scale/')

# single-step downscaled

m_exc1_1step, m_exc2_1step, m_inh1_1step, m_inh2_1step = loaddata(binarydatapath + 'single_step_downscaled/')

# two-step downscaled

m_exc1_2step, m_exc2_2step, m_inh1_2step, m_inh2_2step = loaddata(binarydatapath + 'two_step_downscaled/')

# van Vreeswijk scaling

m_exc1_vv, m_exc2_vv, m_inh1_vv, m_inh2_vv = loaddata(binarydatapath + 'Vreeswijk_scaling/')



################### Compute f and g' for different network sizes ####################

g = 3.
gamma = 1.
NE = 1000.

epsilon = 1./numpy.sqrt(NE)
j_EE = 3.*epsilon    # coupling strength E->E
j_EI = -g*j_EE  # coupling strength I->E
j_IE = j_EE   # coupling strength E->I
j_II = -g*j_EE  # coupling strength I->I
j_IX = j_EE
j_EX = j_EE
p = 0.2

J_array = numpy.array([[j_EE, j_EI, j_EX], [j_IE, j_II, j_IX]])
K_array = numpy.array([[p*NE, gamma*p*NE, p*NE], [p*NE, gamma*p*NE, p*NE]])
# mean activity of external neurons
m_x = 0.
# threshold
theta = -3.0

network = ginzburg(numpy.array([NE,gamma*NE,NE]), J_array, K_array, m_x, theta)

ms, cs = network.solve_self_consistently_corr(numpy.array([0.3, 0.3]), m_x, 0)

a = ms[0]*(1-ms[0]) # ms[0] and ms[1] are identical
mu = p*NE*j_EE*ms[0] + gamma*p*NE*j_EI*ms[1]
sigma = numpy.sqrt(p*NE*j_EE**2*a*(1+gamma*g**2))
c_ee = cs[0] 
c_ii = cs[2] 

# c_ee and c_ii solve Grytskyy et al. - Invariance of covariances arises out of noise, Eq. 7, but note that the N there is the nr of excitatory neurons only.

q = p*NE*j_EE/(numpy.sqrt(2*numpy.pi)*sigma)*numpy.exp(-(mu-theta)**2/(2*sigma**2))

def f_and_g(N):
  f_new = 1./(q*((a/N+c_ee)*(a/N+gamma*c_ii) - gamma/4.*(c_ee+c_ii)**2))*(a/N*c_ee + gamma/2.*c_ii*(c_ee - c_ii))
  g_new = (c_ee*(c_ee-c_ii)-2.*a/N*c_ii)/(gamma*c_ii*(c_ee-c_ii) + 2.*a/N*c_ee)
  return f_new, g_new

N_rel = numpy.arange(0.01,2,0.01)

f_new = numpy.zeros(len(N_rel))
g_new = numpy.zeros(len(N_rel))

for i in xrange(len(N_rel)):
  f_new[i], g_new[i] = f_and_g(N_rel[i]*NE)
  print 'L: ', f_new[i]*q*(1.-g_new[i]*gamma)


################### Plot ####################

scale = 1.
width = 6.83 
n_horz_panels = 2
n_vert_panels = 3

panel_factory = create_fig_plos(1, scale, width, n_horz_panels, n_vert_panels)
axA = panel_factory.new_empty_panel(0, 0, r'A')
# diagram showing redistribution of synapses

axB = panel_factory.new_panel(1, 0, r'B')
# f, g
pl.plot(N_rel, f_new, color=mycols2[0])
pl.plot(N_rel, g_new, color=mycols2[1])
pl.legend(['f', 'g'], loc=2)
pl.xlim([0,1.12])
pl.ylim([-1,5])
pl.xlabel('$N/N_0$')
pl.plot([0, 0.38],[1, 1], '--k', dashes=(4,7))
pl.plot([0.38, 0.38],[-1, 1], '--k', dashes=(4,7))

axC = panel_factory.new_panel(0, 1, r'C')
# mean activities of excitatory neurons vs time 
m_exc_full = 0.5 * (m_exc1_full+m_exc2_full) / N_rec_full
m_exc_1step = 0.5 * (m_exc1_1step+m_exc2_1step) / N_rec_1step
m_exc_2step = 0.5 * (m_exc1_2step+m_exc2_2step) / N_rec_2step
m_exc_vv = 0.5 * (m_exc1_vv+m_exc2_vv) / N_rec_vv

i0 = int(1000./dt_samp)
ts = numpy.arange(len(m_exc1_vv[i0:]))*dt_samp # dt_samp should be the same for all sets of parameters

pl.plot(ts, m_exc_full[i0:], color=mycols[0][0], label=r'full scale')
pl.plot(ts, m_exc_vv[i0:], color=mycols[0][2], label=r'conventional scaling')

pl.xlabel(r'$t (\mathrm{ms})$')
pl.ylabel(r'avg. activity $n_{\mathrm{e}}$')

pl.xlim([1000., 2000.])
pl.ylim([0, .22])
pl.xticks([1000., 1500., 2000.])

pl.legend(loc=4)


axD = panel_factory.new_panel(1, 1, r'D')
# covariances

print 'full-scale correlations'
cc_ee_full, cc_ei_full, cc_ii_full = correlate(m_exc1_full, m_exc2_full, m_inh1_full, m_inh2_full, N_rec_full)

print 'one-step downscaled correlations'
cc_ee_1step, cc_ei_1step, cc_ii_1step = correlate(m_exc1_1step, m_exc2_1step, m_inh1_1step, m_inh2_1step, N_rec_1step)

print 'two-step downscaled correlations'
cc_ee_2step, cc_ei_2step, cc_ii_2step = correlate(m_exc1_2step, m_exc2_2step, m_inh1_2step, m_inh2_2step, N_rec_2step)

print 'van Vreeswijk correlations'
cc_ee_vv, cc_ei_vv, cc_ii_vv = correlate(m_exc1_vv, m_exc2_vv, m_inh1_vv, m_inh2_vv, N_rec_vv)

tlag = numpy.arange(len(cc_ee_vv))*dt_samp - len(cc_ee_vv)/2*dt_samp # dt_samp should be the same for all sets of parameters

f = 1e5
pl.plot(tlag, f*cc_ee_full, color=mycols[0][0], label=r'$c_{\mathrm{ee}}$')
pl.plot(tlag, f*cc_ei_full, color=mycols[1][0], label=r'$c_{\mathrm{ei}}$')
pl.plot(tlag, f*cc_ii_full, color=mycols[2][0], label=r'$c_{\mathrm{ii}}$')

pl.plot(tlag, f*cc_ee_vv, color=mycols[0][2])
pl.plot(tlag, f*cc_ei_vv, color=mycols[1][2])
pl.plot(tlag, f*cc_ii_vv, color=mycols[2][2])

# indicate zero-lag covariances
zero = numpy.where(tlag==0)[0]
pl.plot(0, f*cc_ee_full[zero], 'x', color=mycols[0][0], markersize=10)
pl.plot(0, f*cc_ei_full[zero], 'x', color=mycols[1][0], markersize=10)
pl.plot(0, f*cc_ii_full[zero], 'x', color=mycols[2][0], markersize=10)

pl.plot(0, f*cc_ee_vv[zero], '.', color=mycols[0][2], markersize=10)
pl.plot(0, f*cc_ei_vv[zero], '.', color=mycols[1][2], markersize=10)
pl.plot(0, f*cc_ii_vv[zero], '.', color=mycols[2][2], markersize=10)

pl.legend(loc=1)

pl.xlabel(r'$\Delta (\mathrm{ms})$')
pl.ylabel(r'$c \; (10^{-5})$')

tmin, tmax = -25., 25.
pl.xlim([tmin, tmax])

axE = panel_factory.new_panel(0, 2, r'E')
pl.plot(ts, m_exc_full[i0:], color=mycols[0][0], label=r'full scale')
pl.plot(ts, m_exc_1step[i0:], color=mycols[0][2], label=r'downscaled')
pl.xlabel(r'$t (\mathrm{ms})$')
pl.ylabel(r'avg. activity $n_{\mathrm{e}}$')
pl.xlim([1000., 2000.])
pl.ylim([0, .22])
pl.xticks([1000., 1500., 2000.])
pl.legend(loc=4)

axF = panel_factory.new_panel(1, 2, r'F')
pl.plot(tlag, f*cc_ee_full, color=mycols[0][0], label=r'$c_{\mathrm{ee}}$')
pl.plot(tlag, f*cc_ei_full, color=mycols[1][0], label=r'$c_{\mathrm{ei}}$')
pl.plot(tlag, f*cc_ii_full, color=mycols[2][0], label=r'$c_{\mathrm{ii}}$')
pl.plot(tlag, f*cc_ee_1step, color=mycols[0][2])
pl.plot(tlag, f*cc_ei_1step, color=mycols[1][2])
pl.plot(tlag, f*cc_ii_1step, color=mycols[2][2])
pl.xlabel(r'$\Delta (\mathrm{ms})$')
pl.ylabel(r'$c \; (10^{-5})$')
pl.xlim([tmin, tmax])
pl.ylim([-10,40])

# indicate zero-lag covariances
pl.plot(0, f*cc_ee_full[zero], 'x', color=mycols[0][0], markersize=10)
pl.plot(0, f*cc_ei_full[zero], 'x', color=mycols[1][0], markersize=10)
pl.plot(0, f*cc_ii_full[zero], 'x', color=mycols[2][0], markersize=10)

pl.plot(0, f*cc_ee_1step[zero], '.', color=mycols[0][2], markersize=10)
pl.plot(0, f*cc_ei_1step[zero], '.', color=mycols[1][2], markersize=10)
pl.plot(0, f*cc_ii_1step[zero], '.', color=mycols[2][2], markersize=10)

pl.legend(loc=1)

figurename = "binary_network_scaling_symm"

savefig(figurepath + figurename + '.eps')

# merge the drawing in panel A
merge_inkscape_eps_into_matplotlib_eps(figurepath + figurename + '.eps', figurepath + 'redistributing_synapses.eps', figurepath + 'fig4.eps', (35., 300.), (4.8, 4.8))

