# Sacha van Albada, Nov 2014

# Script for plotting results of scaling an E-I LIF network
# with population-independent connection probability

# Data created using hambach_run_g_gamma_K_scaling.py

import siegert as sig
import numpy
from plotfuncs import *
from pylab import *
import hawkesnet as hn
from corr_hawkes_like import *
from ps_manip import *

figurepath = '../figures/'
datapath = '../data/spiking/E_I_LIF_network/'

mycols = [['#1d44b8', '#4169e1', '#839deb'], # blue from dark to light
          ['#6533cb', '#9370db', '#c1adea'], # purple
          ['#af143c', '#c75a76', '#dfa1b1']] # pinkish red

mycols2 = [black, '#1d771d', '#a8d57b']

####### Determine K, g, gamma as a function of NE  ########

NE0 = 10000
J = 0.1
mu = 15.0   # mV
sigma = 10.0 # mV
nu0 = 23.6   # Hz rate in network
gamma0  = 0.25
epsilon0 = 0.1 # connection probability
tauMem = 20.0
tauRef = 2.0 # ms
tauSyn = 2.0 # ms
CMem = 1.0
theta  = 15.0
V_r = 0.0
g0 = 5.0
J_ext = 0.1

# Everything vs N'/N:
w0 = sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, J)[0]

# N at which scaled quantities diverge
Nlim = NE0*(epsilon0*NE0*w0*(2*g0+1.-gamma0*g0**2)-2*g0)/(epsilon0*NE0*w0*(1+g0)**2)
print 'Nlim: ', Nlim

def scale(epsilon0, w0, g0, gamma0, NE0, NE):
  # Take into account that numbers of neurons and synapses can only be integers
  NI0 = int(round(NE0*gamma0))
  KE0 = int(round(epsilon0*NE0))
  KI0 = int(round(epsilon0*NI0))
  L = w0*(KE0 - g0*KI0)
  KE = int(round(KE0*NE*(2+w0*g0*(KE0+KI0))/(NE0*(2.-L)+(KE0*w0*NE*(1.+g0)))))
  g = (1.+g0)*KE0/KE*NE/NE0 - 1.
  gamma = (KE*w0 - L)/(KE*w0*g)
  epsilon = float(KE)/NE
  NI = int(round(gamma*NE))
  KI = int(round(epsilon*gamma*NE))
  N = NE + NI
  Nsyn = (KE + KI)*N
  return N, KE, KI, Nsyn, g, gamma, epsilon

NE_rel = numpy.arange(0.01,2.01,0.01)

N = numpy.zeros(len(NE_rel))
KE = numpy.zeros(len(NE_rel))
KI = numpy.zeros(len(NE_rel))
Nsyn = numpy.zeros(len(NE_rel))
epsilon = numpy.zeros(len(NE_rel))
g = numpy.zeros(len(NE_rel))
gamma = numpy.zeros(len(NE_rel))

for i in xrange(len(NE_rel)):
  N[i], KE[i], KI[i], Nsyn[i], g[i], gamma[i], epsilon[i] = scale(epsilon0, w0, g0, gamma0, NE0, int(round(NE_rel[i]*NE0)))

NE = NE0*NE_rel

####### Determine external drive that keeps rates constant #######

def nu_exc_inh_from_mu_sigma(w, g, mu, sigma):
  nu_exc_0 = mu / (w*tauMem)
  nu_bal = ((sigma**2 / w**2) - mu/w) / (1.0 + g) / tauMem
  nu_exc = nu_exc_0 + nu_bal
  nu_inh = nu_bal / g  
  return (1000.0*nu_exc, 1000.0*nu_inh)

nu_E_ext = numpy.zeros(len(NE))
nu_I_ext = numpy.zeros(len(NE))

for i in xrange(len(NE)):
  sigma2_local = J**2 * tauMem/1000.*nu0 * epsilon[i]*NE[i] * (1. + gamma[i]*g[i]**2)
  sigma_local = numpy.sqrt(sigma2_local)
  nu_E_ext[i], nu_I_ext[i]  = nu_exc_inh_from_mu_sigma(J_ext, g[i], 0.0, numpy.sqrt(sigma**2-sigma_local**2))

     
####### Determine correlations from theory and simulations #######

def get_corr_data(filename, n1, n2, T, tmax, tbin):
    """Get normalized correlation data"""
    
    corr_data = numpy.loadtxt(filename)
    corr = corr_data[2:]
    count1 = corr_data[0]
    count2 = corr_data[1]
    T_eff = T - tmax    # used new feature of correlation detector to start recording after tmax to avoid triangular fall-off to the sides
    rate1 = count1 / (n1*T_eff/1000.0)
    rate2 = count2 / (n2*T_eff/1000.0)
    rate = numpy.sqrt(rate1*rate2)
    print "rate = ", rate
    corr = 1.0*corr/(n1*n2)/T_eff*1000.0/tbin*1000.0 / (rate1*rate2) - 1.

    return corr, rate

def get_corr(T, N_rec, tmax, tbin, datapath, suffix):
    """Plot simulated correlations"""

    corr_ee, rate_e = get_corr_data(datapath + 'corr_ee' + suffix, N_rec, N_rec, T, tmax, tbin)
    corr_ei, rate_ei = get_corr_data(datapath + 'corr_ei' + suffix, N_rec, N_rec, T, tmax, tbin)
    corr_ii, rate_i = get_corr_data(datapath + 'corr_ii' + suffix, N_rec, N_rec, T, tmax, tbin)
    corr_mm, rate_m = get_corr_data(datapath + 'corr_pp' + suffix, N_rec, N_rec, T, tmax, tbin)

    return corr_ee, corr_ii, corr_ei, corr_mm, rate_e, rate_i, rate_m

def int_corr_theory(NE, g, gamma, epsilon):
    # effective synaptic weight
    w = sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, J)[0]
    gw = sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, -g*J)[0]

    print 'effective weights = ', w, gw

    # effective time constant according to Fourcaud
    
    print 'effective time constant = ', tau_e
    print 'delay = ', delay

    chl = corr_hawkes_like(tau_e, delay, gamma, -gw/w, epsilon, NE, w, nu0/1000., sigma_a, alpha, 30)

    C = chl.int_cov_matrix_corrtrans()

    C_ee = C[0,0]/(nu0/1000.)**2
    C_ei = C[0,1]/(nu0/1000.)**2
    C_ie = C[1,0]/(nu0/1000.)**2
    C_ii = C[1,1]/(nu0/1000.)**2

    return C_ee, C_ei, C_ie, C_ii

def corr_theory_hawkes():
    # effective synaptic weight

    NE = NE0
    NI = round(gamma0*NE0)

    w = sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, J)[0]
    gw = sig.d_nu_d_nu_in_fb(tauMem, tauSyn, tauRef, theta, V_r, mu, sigma, -g0*J)[0]

    print 'effective weights = ', w, gw

    # effective time constant according to Fourcaud
    
    print 'effective time constant = ', tau_e
    print 'delay = ', delay

    h_ana = hn.hawkesnet(NE, NI, epsilon0, g0, w, tau_e, delay, 0.0, 30)
    C = h_ana.cross_cov_matrix2(w, g0, nu0)
    Cs = [C(t) for t in ts_ana]

    C_ee = numpy.array([C[0,0]/(nu0)**2 for C in Cs])
    C_ei = numpy.array([C[0,1]/(nu0)**2 for C in Cs])
    C_ie = numpy.array([C[1,0]/(nu0)**2 for C in Cs])
    C_ii = numpy.array([C[1,1]/(nu0)**2 for C in Cs])

    return C_ee, C_ei, C_ie, C_ii

delay = 0.1

sigma_a = 8.56 # determined from integral of autocorr
tau_e = 4.07   # determined from rate response

alpha = 1.0

procs = [24, 24, 24, 24, 24, 24, 48]
nodes = [1, 1, 1, 1, 1, 1, 2]
Nrecs =[1000, 1000, 1000, 1000, 1000, 1000, 1000]
NEs = [2500, 3500, 5000, 7100, 10000, 14100, 20000]
mem = [1250, 1750, 2500, 3550, 5000, 7050, 10000]

gs = [0.97889182058047486, 1.5149700598802394, 2.3185840707964607, 3.4467640918580376, 5., 7.1976744186046506, 10.363636363636363]
gammas = [1.358490566037736, 0.85770750988142286, 0.55057251908396931, 0.36583888552392485, 0.25, 0.17259019924609587,  0.11933479532163745]
epsilons = [0.30320000000000003, 0.23857142857142857, 0.18079999999999999, 0.13492957746478873, 0.1, 0.073191489361702125, 0.0528]

bplot_indices = [2,4,6]
Nstr = [r"5,\!000", r"10,\!000", r"20,\!000"]

dt      = 0.1      # resolution in ms
simtime = 100000.0 # simulation time in ms

# correlation detector parameters
tbin = dt
tmax = 100.0

i0 = int((tmax - 20.)/tbin)
i1 = int((tmax + 20.)/tbin)

xmin = -20.
xmax = 30.
ymin = -1
ymax = 6
xtcks = [-20., 0., 20.]
ytcks = [0.0, 5., 10.]
f = 100. # scaling of y axis

ts = numpy.arange(-tmax, tmax+tbin, tbin)

ts_ana = arange(-50., 50., 0.1)

int_corr_EE = numpy.zeros(len(NEs))
int_corr_II = numpy.zeros(len(NEs))
int_corr_EI = numpy.zeros(len(NEs))

corr_ees = []
corr_iis = []
corr_eis = []

for i,thisNE in enumerate(NEs):
    suffix = 'NE' + str(thisNE) + 'J' + str(J) + 'mu' + str(mu) + 'sigma' + str(sigma) + 'delay' + str(delay) + '.dat'
    # cross-corr
    corr_ee, corr_ii, corr_ei, corr_mm, rate_e, rate_i, rate_m = get_corr(simtime, Nrecs[i], tmax, tbin, datapath, suffix)
    print 'NE: ', thisNE
    print 'rate_e: ', rate_e
    print 'rate_i: ', rate_i
    int_corr_EE[i] = sum(corr_ee[i0:i1])*tbin
    int_corr_II[i] = sum(corr_ii[i0:i1])*tbin
    int_corr_EI[i] = sum(corr_ei[i0:i1])*tbin
    corr_ees.append(corr_ee)
    corr_iis.append(corr_ii)
    corr_eis.append(corr_ei)


C_ee, C_ei, C_ie, C_ii = corr_theory_hawkes()

iC_ee = numpy.zeros(len(gs))
iC_ei = numpy.zeros(len(gs))
iC_ie = numpy.zeros(len(gs))
iC_ii = numpy.zeros(len(gs))

for i in xrange(len(iC_ee)):
  iC_ee[i], iC_ei[i], iC_ie[i], iC_ii[i] = int_corr_theory(NEs[i], gs[i], gammas[i], epsilons[i])



################### Plot ####################

scale = 1.
width = 6.83
n_horz_panels = 2
n_vert_panels = 3

panel_factory = create_fig_plos(1, scale, width, n_horz_panels, n_vert_panels)
axA = panel_factory.new_empty_panel(0, 0, r'A')
# diagram illustrating scaling


axB = panel_factory.new_panel(1, 0, r'B')

plot(NE_rel, KE/100., color=mycols2[0])
plot(NE_rel, g, color=mycols2[1])
# Plot the positive and negative branches of gamma separately
plot(NE_rel[gamma<0], gamma[gamma<0], color=mycols2[2])
plot(NE_rel[gamma>0], gamma[gamma>0], color=mycols2[2])
legend([r"$K' (10^2)$", r"$g'$", r"$\gamma'$"], loc=4)
#xlim([0,1.25])
ylim([-30,20])
xlabel("$N_1/N$")
# g*gamma remains > 1 for the whole range
# gamma and g become negative at the same point
N_g0 = NE_rel[abs(g) == min(abs(g))]
print 'NE_rel where g is closest to 0: ', N_g0
plot([0, N_g0],[0, 0], '--k')
plot([N_g0, N_g0], [min(ylim()),max(ylim())], '--k')


axC = panel_factory.new_panel(0, 1, r'C')
# Total numbers of neurons and synapses vs NE
plot(NE_rel, Nsyn/1e7, color=mycols2[1])
plot(NE_rel[gamma<0], N[gamma<0]/1e4, color=mycols2[2])
plot(NE_rel[gamma>0], N[gamma>0]/1e4, color=mycols2[2])
xlabel("$N_1/N$")
ylim([-2,5])
legend(['$N_{\mathrm{syn}} (10^7)$', '$N_{\mathrm{total}} (10^4)$'], loc=1)
plot([N_g0, N_g0], [min(ylim()),max(ylim())], '--k')


axD = panel_factory.new_panel(1, 1, r'D')
# external drive vs NE
plot(NE_rel, nu_E_ext/1.e6, color=mycols[0][0])
plot(NE_rel[gamma<0], nu_I_ext[gamma<0]/1.e6, color=mycols[2][0])
plot(NE_rel[gamma>0], nu_I_ext[gamma>0]/1.e6, color=mycols[2][0])
xlabel("$N_1/N$")
legend([r'$r_{\mathrm{e, ext}}$', r'$r_{\mathrm{i, ext}}$'], loc=1)
ylim([-5, 4])
ylabel(r'$r_{\mathrm{ext}}~(10^6~\mathrm{spikes/s})$')
plot([N_g0, N_g0], [min(ylim()),max(ylim())], '--k')


axE = panel_factory.new_panel(0, 2, r'E')
# integrated correlations from theory and simulations
loglog(NEs, int_corr_EE, 'x', color=mycols[0][0])
loglog(NEs, int_corr_EI, 'x', color=mycols[1][0])
loglog(NEs, int_corr_II, 'x', color=mycols[2][0])

loglog(NEs, iC_ee, '.', color=mycols[0][0])
loglog(NEs, iC_ei, '.', color=mycols[1][0])
loglog(NEs, iC_ii, '.', color=mycols[2][0])

rcParams['legend.fontsize'] = scale*10
legend()

xlim([0.8*NEs[0], 1.2*NEs[-1]])
xticks([1e4,2e4], [r'$10^4$', r'$2\times10^4$'])
xlabel(r"$N_1$")
ylabel(r'$C(0)$')
ylim([0.01, 5])
legend([r'$C_\mathrm{ee}(0)$', r'$C_\mathrm{ei}(0)$', r'$C_\mathrm{ii}(0)$'])



axF = panel_factory.new_panel(1, 2, r'F')
# EI correlation function vs time lag
for i,corr_ei in enumerate(numpy.array(corr_eis)[bplot_indices]):
    plot(ts, f*corr_ei, linewidth=1., label=r"$N_1=" + Nstr[i] + '$', color=mycols[1][2-i])

plot(ts_ana, f*C_ie, '--', color='k')

rcParams['legend.fontsize'] = scale*10
legend()

xticks([])
yticks(ytcks)
xlim([xmin, xmax])
ylim([ymin, ymax])
xticks(xtcks)
xlabel(r'$\Delta \; (\mathrm{ms})$')
ylabel(r'$\frac{c_\mathrm{ei}}{r_\mathrm{e} r_\mathrm{i}} (10^{-2})$')


figurename = "spiking_network_scaling"

savefig(figurepath + figurename + '.eps')

# merge the drawing in panel A
merge_inkscape_eps_into_matplotlib_eps(figurepath + figurename + '.eps', figurepath + 'spiking_network_scaling_diagram.eps', figurepath + 'fig5.eps', (50., 300.), (4.8, 4.8))


