import h5py
import numpy

############################################################
def add_to_h5(filename,dict):
    ''' 
    Stores a dictionary (dict), in a file (filename).

    Inputs:
      filename - a string, name of file to store the dictionary
      dict     - a dictionary, one or several pairs of string and any type of variable,
                 e.g. dict = {'name1': var1,'name2': var2}
    '''
    f = h5py.File(filename,'w')
    for k in dict:
        f[k]=dict[k]
    f.close()

############################################################
def load_h5(filename):
    '''
    Loads h5-file and extracts the dictionary within it. 

    Outputs:
      dict - dictionary, one or several pairs of string and any type of variable,
             e.g dict = {'name1': var1,'name2': var2} 
    '''

    f = h5py.File(filename,'r')
    dict = {}
    for k,v in zip(f.keys(),f.values()):
        dict[k]=numpy.array(v[:])
    f.close()
    return dict

############################################################
def test():
    filename='test.h5'
    d={}
    d['a']=numpy.array([1,2,3])
    d['b']=numpy.arange(0.,0.5,0.01)

    add_to_h5(filename,d)
    dd=load_h5(filename)

