# Sacha van Albada, Apr 2015

# Script for running LIF_network_rate_comparison.py on a cluster
# Adjust to your system as necessary

import os
import glob
import numpy
from parameters_LIF_network_rate_comparison import *

procs = 24
nodes = 1
mem = 5000

NE_full = 8000

nu = [3.3, 29.6, 3.3, 29.6] # from full-scale simulations
mu_ext_full = [10., 25., 10., 25.]
sigma_ext_full = [5., 20., 5., 20.]

simname = 'LIF_network_rate_comparison'

workingdir = '/users/albada/Documents/downscale_manuscript/submission2/py'
qsub_base = '/users/albada/Documents/downscale_manuscript/submission2/data/qsub'

# Scaling factors. The minimal in-degree scaling factor is given by var_int/(var_int+var_ext)
K_scaling = [0.5, 0.5, 0.5, 0.5]
N_scaling = [1, 1, 0.5, 0.5]

os.system('cd ' + workingdir)

# create one pyf and one jdf file per job

for k in xrange(len(mu_ext_full)):
    for scale_type in ['vreeswijk', 'helias']:
        qsubfile = qsub_base + '_' + str(k) +  '_' + scale_type
        # write a python file to execute
        pyf_name = './pyf_' + str(k) + '_' + scale_type + '.py'
        f = open( pyf_name , 'w')
        f.write( 'import LIF_network_rate_comparison\n' )
        f.write( 'LIF_network_rate_comparison.simulate('
                       +'"' + scale_type + '",'\
                       +str(K_scaling[k])+','\
                       +str(N_scaling[k])+','\
                       +str(g)+','\
                       +str(gamma)+','\
                       +str(epsilon)+','\
                       +str(J)+','\
                       +str(NE_full)+','\
                       +str(delay)+','\
                       +str(nu[k])+','\
                       +str(mu_ext_full[k])+','\
                       +str(sigma_ext_full[k])+','\
                       +str(int(N_scaling[k]*N_rec))+','\
                       +str(procs) +') \n' )

        f.close()
      
        # write a job description file for qsub
        jdf_name = './job_' + str(k) + '_' + scale_type + '.jdf' 
        f = open( jdf_name , 'w')
        f.write( '#!/bin/bash \n')
        f.write( '#PBS -N '+simname+'_'+str(k)+'\n' )  #set job name
        f.write( '#PBS -d '+workingdir+'\n' )    #working directory
        f.write( '#PBS -o '+qsubfile+'\n' )      #output directory to send standard/error output stream to
        f.write( '#PBS -j oe \n' )               #join error stream and output stream to obtain a single file in the end
        f.write( '#PBS -l nodes=' + str(nodes) + ':ppn=24\n' )    #request a total of nodes*8 processors for this job
        f.write( '#PBS -l walltime=24:00:00,mem=' + str(mem) + 'mb \n' ) #request processor time and memory
        f.write('export PYTHONPATH=/users/albada/10kcollaps_install.hambach/lib64/python2.6/site-packages\n')
        f.write('. /usr/local/mpi/openmpi/1.4.3/gcc64/bin/mpivars_openmpi-1.4.3_gcc64.sh\n') 
        f.write('mpirun -np ' + str(procs) + ' python ' + pyf_name + '\n')
        f.close()
       
        # submit job
        os.system('qsub ' + jdf_name)


