# Sacha van Albada, Nov 2014

# Code for running simulation of network of inhibitory 
# LIF neurons with exponential current-based synapses
# with two different sizes, and 1/K or 1/sqrt(K) scaling
# on a cluster.
# The connection probability epsilon is fixed.

# Adjust to your system as necessary.

import os
import glob
import numpy


##################################################
# queue specific

nodes = 1          # how many nodes to use
mpiprocs = 24      # how many processes to use
core_per_node = 24 # how many processors per node
threads_per_process = 1

workingdir = '/scratch/albada/inhibitory_LIF_network/py'

wallclock_limit = "20:00:00"

os.system('cd ' + workingdir)

nvp = nodes*core_per_node
mem_tot = 2.2*nvp

# size of reference network
N0 = 10000

# size of up-scaled networks
N = 50000

os.system('cd ' + workingdir)

for scale_type in ['none', 'vreeswijk', 'helias']:

    qsubfile = '/scratch/albada/inhibitory_LIF_network/qsub' + '_' + scale_type

    simname = 'inhibitory_LIF_network_' + scale_type

    # write a python file to execute

    pyf_name = './pyf_' + str(N) + '_' + scale_type + '.py'
    f = open( pyf_name , 'w')
    f.write('import inhibitory_LIF_network as iln\n')
    f.write('iln.simulate(' + str(N0) + ',' + str(N) + ',"' + scale_type + '",' + str(nvp) + ')' )
    f.close()

    # write a job description file for qsub

    jdf_name = './job_' + str(N) + '_' + scale_type + '.jdf' 
    f = open( jdf_name , 'w')
    f.write( '#!/bin/bash \n')
    f.write( '#PBS -N '+simname+'\n' )  #set job name
    f.write( '#PBS -d '+workingdir+'\n' )    #working directory
    f.write( '#PBS -o '+qsubfile+'\n' )      #output directory to send standard/error output stream to
    f.write( '#PBS -j oe \n' )               #join error stream and output stream to obtain a single file in the end
    f.write( '#PBS -l nodes=' + str(nodes) + ':ppn='+str(core_per_node)+'\n' )    #request a total of nodes*proc_per_node processors for this job
    f.write( '#PBS -l mem=%dGB\n' % mem_tot )    #request a total of nodes*proc_per_node processors for this job
    f.write( '#PBS -l walltime=' + wallclock_limit + '\n' ) #request for processor time
    f.write('export PYTHONPATH=/users/albada/10kproject_install.hambach/lib64/python2.6/site-packages\n')
    f.write('. /usr/local/mpi/openmpi/1.4.3/gcc64/bin/mpivars_openmpi-1.4.3_gcc64.sh\n') 
    f.write('mpirun -np ' + str(mpiprocs) + ' -bynode -bind-to-core -cpus-per-proc ' + str(threads_per_process) + ' -report-bindings ' + 'python ' + pyf_name + '\n')


    f.close()
       
    # submit job
    os.system('qsub ' + jdf_name)
