# Sacha van Albada, Nov 2014

from network_params import *
import random
from scipy import stats
import numpy as np
from pyNN.random import RandomDistribution

def MultinomialConnect(sim, pop1, pop2, K, weight_mean, weight_sd, delay_mean, delay_sd):
    if not K:
        return
    source_neurons = list(pop1.all_cells)
    target_neurons = list(pop2.all_cells)
    n_syn = int(round(K*len(target_neurons)))
    delay_dict = {'distribution': 'normal_clipped',
                  'mu': delay_mean,
                  'sigma': delay_sd,
                  'low': 0.1,
                  'high': np.inf}
    if weight_mean > 0:
        if lognormal_weights:
            mu = np.log(1000*weight_mean/(np.sqrt(1. + weight_sd**2)))
            sigma = np.sqrt(np.log(1 + weight_sd**2))
            weight_dict = {'distribution': 'lognormal',
                           'mu': mu,
                           'sigma': sigma}
        else:
            weight_dict = {'distribution': 'normal_clipped',
                           'mu': 1000*weight_mean,
                           'sigma': 1000*weight_sd,
                           'low': 0.,
                           'high': np.inf}
    else:
        if lognormal_weights:
            mu = -np.log(-1000*weight_mean/(np.sqrt(1. + weight_sd**2)))
            sigma = np.sqrt(np.log(1 + weight_sd**2))
            weight_dict = {'distribution': 'lognormal',
                           'mu': mu,
                           'sigma': sigma}
        else:            
            weight_dict = {'distribution': 'normal_clipped',
                           'mu': 1000*weight_mean,
                           'sigma': 1000*weight_sd,
                           'low': -np.inf,
                           'high': 0.}

    syn_dict = {'weight': weight_dict,
                'delay': delay_dict}

    conn_dict = {'rule': 'fixed_total_number',
                 'N': n_syn}

    sim.nest.sli_push(source_neurons)
    sim.nest.sli_push(target_neurons)
    sim.nest.sli_push(conn_dict)
    sim.nest.sli_push(syn_dict)
    sim.nest.sli_run("Connect")

